import click
import requests
import sys
import numpy as np
from pathlib import Path
import uuid
from stochasticx.constants.urls import CloudRoutes, get_cloud_url, LocalRoutes

from stochasticx.utils.docker import (
    start_container,
    stop_and_remove_container,
    get_logs_container,
    auto_port_selection,
)

from stochasticx.datasets.datasets import Datasets
from stochasticx.stable_diffusion.download_models import download_model_from_s3
from stochasticx.utils.logging import configure_logger
from stochasticx.deployment.deployments import (
    StableDiffusionDeployments,
    StableDiffusionDeployment,
)

from stochasticx.utils.preferences import AppModes, Preferences
from stochasticx.constants.docker_images import DockerImages, ContainerNames
from stochasticx.utils.auth_utils import AuthUtils
from stochasticx.utils.stat_controller import EventLogger


# from stochasticx.utils.gpu_utils import is_nvidia_gpu_available


@click.group(name="local")
def local():
    try:
        AuthUtils.get_auth_headers()
    except:
        sys.exit()
    EventLogger.log_event("local")


@click.command(name="init")
@click.option(
    "--port", default=3000, help="Port which Stochastic local container will run on"
)
def init(port):
    try:
        AuthUtils.get_auth_headers()
    except:
        sys.exit()

    click.secho(
        "[+] Starting stochasticx in your local environment", fg="blue", bold=True
    )
    click.secho(
        "[+] If it is the first time you deploy the model, it might take some minutes to deploy it",
        fg="blue",
        bold=True,
    )

    original_port = port
    port, is_replace = auto_port_selection(port, ContainerNames.LOCAL)

    # for 2nd port
    port2, _ = auto_port_selection(5432, ContainerNames.LOCAL, selected_ports=[port])
    if is_replace:
        click.secho(
            f"[+] Port {original_port} is using on another service, port {port} will be selected to run stochasticx",
            fg="blue",
            bold=True,
        )
    start_container(
        docker_image=DockerImages.LOCAL,
        ports={"5432": port2, "3000": port},
        container_name=ContainerNames.LOCAL,
        detach=True,
        gpu=False,
        volumes=["stochasticx:/vol"],
    )

    try:
        click.echo("[+] Setting up preferences...")
        preferences = Preferences.load()
        preferences.current_mode = AppModes.LOCAL
        preferences.local_url = f"http://127.0.0.1:{port}"
        Preferences.save(preferences)
    except:
        stop()

    click.secho(f"[+] x-local server running in the port {port}", fg="green", bold=True)
    # click.echo("[+] Using GPU: {}".format(is_nvidia_gpu_available()))
    EventLogger.log_event("local_init")


@click.command(name="logs")
def logs():
    click.secho("[+] Logs", fg="blue", bold=True)
    logs = get_logs_container(ContainerNames.LOCAL)
    click.secho(logs, fg="white", bold=True)
    EventLogger.log_event("local_logs")


@click.command(name="stop")
def stop():
    click.secho("[+] Stopping and removing x-local", fg="blue", bold=True)
    stop_and_remove_container(ContainerNames.LOCAL)
    click.secho("[+] Removed", fg="green", bold=True)
    EventLogger.log_event("local_stop")


local.add_command(init)
local.add_command(logs)
local.add_command(stop)



import os
from pathlib import Path
import click
from getpass import getpass
from stochasticx.utils.stat_controller import EventLogger
from stochasticx.auth.auth import Stochastic
from stochasticx.constants.urls import TOKEN_AUTH_PATH

stochastic = Stochastic()


@click.command(name="login")
@click.option("--username", help="Username for login")
@click.option("--password", help="Password for login")
def login(username, password):
    if username is None and password is None:
        stochastic.link_login()
    else:
        if username is None:
            username = input("Enter your email: ")

        if password is None:
            password = getpass()

        stochastic.login(username, password)

    click.secho("[+] Login successfully", fg="green", bold=True)
    EventLogger.log_event("login")


@click.command(name="logout")
def logout():
    token_path = Path(TOKEN_AUTH_PATH).resolve()

    if token_path.exists():
        os.remove(token_path)

    click.secho("[+] Logged out successfully", fg="green", bold=True)
    EventLogger.log_event("logout")


@click.group(name="me")
def me():
    pass


@click.command(name="profile")
def profile():
    click.secho("\n[+] Collecting profile information", fg="blue", bold=True)
    profile_info = stochastic.get_profile()
    click.secho(profile_info, bold=True)
    EventLogger.log_event("profile")


@click.command(name="company")
def company():
    company_info = stochastic.get_company()
    click.secho("\n[+] Collecting company information", fg="blue", bold=True)
    click.secho(company_info, bold=True)
    EventLogger.log_event("company")


@click.command(name="usage")
def usage():
    usage_info = stochastic.get_usage_quota()
    click.secho("\n[+] Collecting usage information", fg="blue", bold=True)
    click.secho(usage_info, bold=True)
    EventLogger.log_event("usage")


me.add_command(profile)
me.add_command(company)
me.add_command(usage)



import os

import stochasticx
from stochasticx.utils.preferences import Preferences, AppModes

if os.name == 'nt':
  HOME = "%userprofile%"
else:
  HOME = "$HOME"

TOKEN_AUTH_PATH = os.path.expandvars(f"{HOME}/.stochastic/token.json")
PREFERENCES_PATH = os.path.expandvars(f"{HOME}/.stochastic/preferences.json")

INFERENCE_URL = stochasticx.BASE_URI + "http://infer.stochastic.ai:8000/"
FINETUNING_GET_JOBS_CLOUD_URL = stochasticx.BASE_URI + "/v1/finetuningJobs"

DEPLOYMENTS_URL = stochasticx.BASE_URI + "/v1/deploy/"
STOCHASTIC_TEMPLATES_URL = stochasticx.BASE_URI + "/v1/deploy/stochastic-template"
CUSTOM_TEMPLATES_URL = stochasticx.BASE_URI + "/v1/deploy/custom-template"
IS_INFERENCE_FILE_AVAILABLE_URL = stochasticx.BASE_URI + "/v1/deploy/is-inference-file-available"

class CloudRoutesType:
    def __init__(self, suffix):
        self.suffix = suffix

    def cloud_add(self, url):
        return url + self.suffix

    def formatting(self, id):
        self.suffix += f"/{id}"
        return self


class CloudRoutes:
    LOGIN_URL = CloudRoutesType("/v1/auth/login")
    MODELS_URL = CloudRoutesType("/v1/models")
    OPTIMIZED_MODELS_URL = CloudRoutesType("/v1/processedModels")
    REQUEST_UPLOAD_URL = CloudRoutesType("/v1/upload/requestUpload")
    MODEL_UPLOAD_URL = CloudRoutesType("/v1/upload/model")
    DATASET_UPLOAD_URL = CloudRoutesType("/v1/upload/dataset")
    DATASETS_URL = CloudRoutesType("/v1/datasets")
    ME_URL = CloudRoutesType("/v1/auth/me")
    JOBS_URL = CloudRoutesType("/v1/jobs")
    INSTANCES_URL = CloudRoutesType("/v1/instances")
    DEPLOYMENT_URL = CloudRoutesType("/v1/deploy")
    STABLE_DIFFUSION_URL = CloudRoutesType("/v1/stdDeploy")
    INFERENCE_URL = "http://infer.stochastic.ai:8000/"
    


class LocalRoutesType:
    def __init__(self, suffix):
        self.suffix = suffix

    def local_add(self, url):
        return url + self.suffix


class LocalRoutes:
    HEALTH_REGISTRY = "http://127.0.0.1:3001/"
    LOGIN_URL = LocalRoutesType("/v1/auth/login")
    MODEL_URL = LocalRoutesType("/local/model")
    MODEL_UPLOAD_URL = LocalRoutesType("/local/model/upload")
    DATASET_URL = LocalRoutesType("/local/dataset")
    DATASET_UPLOAD_URL = LocalRoutesType("/local/dataset/upload")
    BENCHMARKING_URL = LocalRoutesType("/post_benchmarking_task")
    ONNX_CONVERSION_URL = LocalRoutesType("/onnx_conversion/")
    TENSORRT_CONVERSION_URL = LocalRoutesType("/tensorrt_conversion/")
    ONNX_INT8_CONVERSION_URL = LocalRoutesType("/onnx_conversion_int8/")


class CommonRoutes:
    LOGIN_URL = "LOGIN_URL"
    MODELS_URL = "MODELS_URL"


def get_cloud_url(suffix: CloudRoutesType) -> str:
    # Read saved preferences
    preferences = Preferences.load()
    return suffix.cloud_add(preferences.cloud_url)


def get_local_url(suffix: LocalRoutesType, type: str) -> str:
    # Read saved preferences
    preferences = Preferences.load()
    return suffix.local_add(preferences.__getattribute__(type))


def get_common_url(suffix: str) -> str:
    # Read saved preferences
    print("Suffix: ", suffix)
    preferences = Preferences.load()
    current_mode = preferences.current_mode

    if current_mode == AppModes.CLOUD:
        return preferences.cloud_url + suffix
    else:
        return preferences.local_url + suffix


# def test():

#     common_url = get_common_url(CommonRoutes.LOGIN_URL)

#     print("CommonURL: ", common_url)

#     cloud_url = get_cloud_url(CommonRoutes.LOGIN_URL)

#     print("CloudURL: ", cloud_url)

#     local_url = get_local_url(CommonRoutes.LOGIN_URL)

#     print("LocalURL: ", local_url)

# test()



from pathlib import Path
import os
import requests
from tqdm import tqdm
import json

import stochasticx

from stochasticx.constants.urls import (
    get_local_url,
    get_cloud_url,
    LocalRoutes,
    CloudRoutes,
)

from stochasticx.utils.auth_utils import AuthUtils


class FileUtils:
    """File utils"""

    @staticmethod
    def calculate_directory_size(directory_path: str):
        """Calculates the total size of a directory

        Args:
            directory_path (str): the directory path

        Returns:
            int: size in bytes
        """
        size = 0
        directory_path = Path(directory_path)

        assert directory_path.exists()

        all_files_dirs = directory_path.glob("**/*")
        for file_or_dir in all_files_dirs:
            if file_or_dir.is_file():
                size += os.path.getsize(str(file_or_dir.resolve()))

        return size


class DownloadUploadUtils:
    """Download utils"""

    @staticmethod
    def _download(prepare_download_path: str, download_path: str, local_path: str):
        """Download request

        Args:
            prepare_download_path (str): prepare download url
            download_path (str): download url
            local_path (str): local path where the file will be saved
        """
        auth_header = AuthUtils.get_auth_headers()

        # Prepare download
        r = requests.get(url=prepare_download_path, headers=auth_header)
        r.raise_for_status()
        download_token = r.json().get("downloadToken")

        # Download model
        with requests.get(
            url=download_path + "?auth_token={}".format(download_token),
            headers=auth_header,
            stream=True,
        ) as r:
            r.raise_for_status()
            total_size_in_bytes = int(r.headers.get("content-length", 0))
            progress_bar = tqdm(total=total_size_in_bytes, unit="iB", unit_scale=True)
            with open(str(local_path), "wb") as f:
                for chunk in r.iter_content(chunk_size=8192):
                    progress_bar.update(len(chunk))
                    f.write(chunk)


class ModelUtils:
    """ModelUtils"""

    @staticmethod
    def download_model(model_id: str, local_path: str):
        """Downloads a model

        Args:
            model_id (str): model ID to be downloaded
            local_path (str): path where the model will be saved
        """
        assert model_id is not None
        prepare_download_path = (
            stochasticx.BASE_URI + "/v1/models/{}/prepareDownload".format(model_id)
        )
        download_path = stochasticx.BASE_URI + "/v1/models/{}/download/".format(
            model_id
        )

        DownloadUploadUtils._download(prepare_download_path, download_path, local_path)

    @staticmethod
    def download_optimized_model(model_id: str, local_path: str):
        """Downloads the optimized model

        Args:
            model_id (str): model ID
            local_path (str): path where the model will be saved
        """

        assert model_id is not None
        prepare_download_path = (
            stochasticx.BASE_URI
            + "/v1/processedModels/{}/prepareDownload".format(model_id)
        )
        download_path = (
            stochasticx.BASE_URI + "/v1/processedModels/{}/download/".format(model_id)
        )

        DownloadUploadUtils._download(prepare_download_path, download_path, local_path)

    @staticmethod
    def upload_model(directory_path: str, model_name: str, model_type: str, mode: str):
        """Uploads a model to the platform

        Args:
            directory_path (str): directory path where the model is located
            model_name (str): model name
            model_type (str): model type

        Returns:
            str: model ID
        """
        assert directory_path is not None
        directory_size = FileUtils.calculate_directory_size(directory_path)

        # Request upload
        if mode == "cloud":
            auth_header = AuthUtils.get_auth_headers()
            data = {
                "folderSize": directory_size,
                "resourceType": "model",
                "resourceName": model_name,
            }
            suffix = CloudRoutes.REQUEST_UPLOAD_URL
            url = get_cloud_url(suffix)

            r = requests.post(url, json=data, headers=auth_header)
            r.raise_for_status()

            url = get_cloud_url(CloudRoutes.MODEL_UPLOAD_URL)
        else:
            auth_header = None
            url = get_local_url(LocalRoutes.MODEL_UPLOAD_URL, "local_url")

        # Upload model
        base_path = Path(directory_path)
        file_list = []
        all_files_dirs = base_path.glob("**/*")

        for file_or_dir_path in all_files_dirs:
            if file_or_dir_path.is_file():
                file = open(str(file_or_dir_path), "rb")
                relative_file_path = str(file_or_dir_path.resolve()).replace(
                    str(base_path.resolve()), ""
                )[1:]

                file_list.append(
                    ("pyModel", (relative_file_path, file, "multipart/form-data"))
                )

        request_body = {
            "name": model_name,
            "type": model_type,
            "folderSize": directory_size,
        }
        r = requests.post(url, data=request_body, files=file_list, headers=auth_header)
        r.raise_for_status()

        model_id = r.json().get("data").get("id")
        return model_id

    @staticmethod
    def add_local_model(directory_path: str, model_name: str, model_type: str):
        """Uploads a model to the platform

        Args:
            directory_path (str): directory path where the model is located
            model_name (str): model name
            model_type (str): model type

        Returns:
            str: model ID
        """
        assert directory_path is not None

        # auth_header = AuthUtils.get_auth_headers()
        auth_header = None

        # Request upload
        directory_size = FileUtils.calculate_directory_size(directory_path)
        # data = {"folderSize": directory_size}
        # print("DATA SIZE: ", data)
        # r = requests.post(, json=data, headers=auth_header)
        # r.raise_for_status()

        # Upload model
        base_path = Path(directory_path)
        file_list = []
        all_files_dirs = base_path.glob("**/*")

        for file_or_dir_path in all_files_dirs:
            if file_or_dir_path.is_file():
                file = open(str(file_or_dir_path), "rb")
                relative_file_path = str(file_or_dir_path.resolve()).replace(
                    str(base_path.resolve()), ""
                )[1:]

                file_list.append(
                    ("pyModel", (relative_file_path, file, "multipart/form-data"))
                )

        request_body = {
            "name": model_name,
            "type": model_type,
            "folderSize": directory_size,
        }
        local_url = get_local_url(LocalRoutes.MODEL_UPLOAD_URL, "local_url")

        r = requests.post(
            local_url, data=request_body, files=file_list, headers=auth_header
        )
        r.raise_for_status()

        # model_id = r.json().get("data")
        model_id = r.json().get("data").get("id")
        return model_id


class DatasetUtils:
    """Dataset utils"""

    @staticmethod
    def download_dataset(dataset_id: str, local_path: str):
        """Download a dataset

        Args:
            dataset_id (str): dataset ID to be downloaded
            local_path (str): local path where the dataset will be saved
        """
        assert dataset_id is not None
        prepare_download_path = (
            stochasticx.BASE_URI + "/v1/datasets/{}/prepareDownload".format(dataset_id)
        )
        download_path = stochasticx.BASE_URI + "/v1/datasets/{}/download/".format(
            dataset_id
        )

        DownloadUploadUtils._download(prepare_download_path, download_path, local_path)

    @staticmethod
    def upload_dataset(
        directory_path: str, dataset_name: str, dataset_type: str, mode: str
    ):
        """Uploads a dataset to the Stochastic platform

        Args:
            directory_path (str): _description_
            dataset_name (str): _description_
            dataset_type (str): _description_

        Returns:
            str: dataset ID
        """
        assert directory_path is not None
        if mode == "cloud":
            auth_header = AuthUtils.get_auth_headers()
        else:
            auth_header = None

        directory_size = FileUtils.calculate_directory_size(directory_path)
        # Request upload
        if mode == "cloud":
            data = {
                "folderSize": directory_size,
                "resourceType": "dataset",
                "resourceName": dataset_name,
            }
            suffix = CloudRoutes.REQUEST_UPLOAD_URL
            url = get_cloud_url(suffix)

            r = requests.post(url, json=data, headers=auth_header)
            r.raise_for_status()

        # Upload dataset
        base_path = Path(directory_path)
        all_files_dirs = base_path.glob("**/*")
        file_list = []

        for file_or_dir_path in all_files_dirs:
            if file_or_dir_path.is_file():
                file = open(str(file_or_dir_path), "rb")
                relative_file_path = str(file_or_dir_path.resolve()).replace(
                    str(base_path.resolve()), ""
                )[1:]
                file_list.append(
                    ("pyDataset", (relative_file_path, file, "multipart/form-data"))
                )

        request_body = {
            "name": dataset_name,
            "type": dataset_type,
            "folderSize": directory_size,
        }
        if mode == "cloud":
            url = get_cloud_url(CloudRoutes.DATASET_UPLOAD_URL)
        else:
            url = get_local_url(LocalRoutes.DATASET_UPLOAD_URL, "local_url")
        r = requests.post(url, data=request_body, files=file_list, headers=auth_header)
        r.raise_for_status()

        dataset_id = r.json().get("data").get("id")
        return dataset_id


class ConversionUtils:
    @staticmethod
    def convert(
        model_path: str,
        convert_type: str,
        model_type: str,
        task_type: str,
        convert_params: dict,
    ):
        """Convert model

        Args:
            model_path (str): directory path where the model is located
            convert_type (str): model type

        Returns:
            str: model ID
        """
        assert model_path is not None

        # auth_header = AuthUtils.get_auth_headers()
        auth_header = None

        # Request upload
        # data = {"folderSize": directory_size}
        # print("DATA SIZE: ", data)
        # r = requests.post(, json=data, headers=auth_header)
        # r.raise_for_status()
        model_path = Path(model_path)
        if os.path.isdir(model_path):
            output_dir = model_path.parent
        else:
            output_dir = model_path

        # local_url = get_local_url(LocalRoutes.MODEL_UPLOAD_URL)
        if convert_type == "onnx":
            request_body = {
                "model_path": str(model_path),
                "output_dir": str(output_dir),
                "model_type": model_type,
                "task_type": task_type,
            }
            local_url = get_local_url(
                LocalRoutes.ONNX_CONVERSION_URL, "local_conversion_url"
            )
        elif convert_type == "tensorrt":
            sequence_length = convert_params.get("sequence_length", 128)
            max_batch_size = convert_params.get("max_batch_size", 1)
            request_body = {
                "model_path": str(model_path),
                "output_dir": str(output_dir),
                "sequence_length": sequence_length,
                "dynamic_batch_size": max_batch_size,
            }
            local_url = get_local_url(
                LocalRoutes.TENSORRT_CONVERSION_URL, "local_conversion_url"
            )
        elif convert_type == "onnx_int8":
            request_body = {
                "model_path": str(model_path),
                "output_dir": str(output_dir),
                "model_type": model_type,
                "task_type": task_type,
            }
            local_url = get_local_url(
                LocalRoutes.ONNX_INT8_CONVERSION_URL, "local_conversion_url"
            )
        r = requests.post(local_url, data=json.dumps(request_body), headers=auth_header)
        return r.json()

        # # model_id = r.json().get("data")
        # model_id = r.json().get("data").get("id")
        # print(model_id)
        # return model_id
        return r



import functools
import json
from datetime import datetime
from typing import Optional
from dataclasses import dataclass, field, asdict
import os
import requests


class GAData:
    measurement_id = "G-M2R5H59FQ6"
    api_secret = "qEYMYZr9Semhw2Eqe046Ig"
    ga_url = f"https://www.google-analytics.com/mp/collect?&measurement_id={measurement_id}&api_secret={api_secret}"


@dataclass
class LocalBenchmarkJobInformation:
    operation: str = "local_benchmark"
    user_info: Optional[str] = None
    session_id: Optional[str] = None
    job_name: Optional[str] = None
    task_name: Optional[str] = None
    task_type: Optional[str] = None
    model_type: Optional[str] = None
    params: Optional[dict] = None
    job_result: Optional[dict] = field(default_factory=dict)


@dataclass
class LocalConversionJobInformation:
    operation: str = "local_conversion"
    user_info: Optional[str] = None
    session_id: Optional[str] = None
    convert_type: Optional[str] = None
    task_type: Optional[str] = None
    model_type: Optional[str] = None
    params: Optional[dict] = None
    job_result: Optional[str] = field(default_factory=str)


@dataclass
class LocalDeploymentJobInformation:
    operation: str = "local_deployment"
    user_info: Optional[str] = None
    session_id: Optional[str] = None
    deployment_type: Optional[str] = None
    model_name: Optional[str] = None
    type: Optional[str] = None
    params: Optional[dict] = None
    job_result: Optional[dict] = field(default_factory=dict)


@dataclass
class LocalFinetuningJobInformation:
    operation: str = "local_finetuning"
    user_info: Optional[str] = None
    session_id: Optional[str] = None
    inference_type: Optional[str] = None
    job_name: Optional[str] = None
    model_name: Optional[str] = None
    dataset_name: Optional[str] = None
    params: Optional[dict] = None
    job_result: Optional[dict] = field(default_factory=dict)


@dataclass
class LocalInferenceJobInformation:
    operation: str = "local_inference"
    user_info: Optional[str] = None
    session_id: Optional[str] = None
    inference_type: Optional[str] = None
    model_name: Optional[str] = None
    model_type: Optional[str] = None
    job_result: Optional[dict] = field(default_factory=dict)


class StatController:
    @classmethod
    def add_information(cls, job_info) -> None:
        """Method to add information to a stat

        Args:
            stat (Stat): stat
            information (str): information
        """
        job_info.user_info = os.getenv("STOCHASTIC_USER")
        job_info.session_id = os.getenv("SESSION_STRING")
        requests.post(url="http://3.226.62.185:8000/log_data", json=asdict(job_info))


class EventLogger:
    @classmethod
    def add_ids(cls, data):
        data["client_id"] = os.getenv("SESSION_STRING")
        data["non_personalized_ads"] = False
        user = os.getenv("STOCHASTIC_USER")

        if user is not None:
            data["user_id"] = user

        return data

    @classmethod
    def log_event(cls, event_name: str) -> None:

        data = {"events": [{"name": event_name}]}

        data = cls.add_ids(data)
        headers = {"content-type": "application/json"}
        result = requests.post(GAData.ga_url, data=json.dumps(data), headers=headers)



from pathlib import Path

import click
import sys
import requests
from stochasticx.utils.preferences import Preferences, AppModes
from stochasticx.constants.urls import LocalRoutes
from stochasticx.datasets.datasets import Datasets, Dataset
from stochasticx.utils.parse_utils import print_table
from stochasticx.utils.auth_utils import AuthUtils
from stochasticx.utils.spinner_slash import Spinner
from stochasticx.utils.stat_controller import EventLogger


@click.group(name="datasets")
def datasets():
    try:
        AuthUtils.get_auth_headers()
    except:
        sys.exit()

    preferences = Preferences.load()

    if preferences.current_mode == AppModes.LOCAL:
        try:
            preferences = Preferences.load()
            response = requests.get(preferences.local_url)
            response.raise_for_status()
        except:
            click.secho(
                "[+] Registry is not initilized. Run the command stochasticx local init",
                fg="red",
                bold=True,
            )
            sys.exit(1)
        EventLogger.log_event("datasets")


@click.command(name="ls")
def ls_datasets():
    preferences = Preferences.load()

    click.secho("\n[+] Collecting all datasets\n", fg="blue", bold=True)
    if preferences.current_mode == AppModes.CLOUD:
        columns, values = Datasets.get_datasets(mode="cloud", fmt="table")
    else:
        columns, values = Datasets.get_datasets(mode="local", fmt="table")
    print_table(columns, values)


@click.command(name="inspect")
@click.option("--id", help="Dataset ID")
def dataset_inspect(id):
    preferences = Preferences.load()

    click.secho("\n[+] Collecting information from the dataset\n", fg="blue", bold=True)
    if preferences.current_mode == AppModes.CLOUD:
        dataset = Datasets.get_dataset("cloud", id)
        if dataset is not None:
            click.echo(dataset.dataset_info)
    else:
        dataset = Datasets.get_dataset("local", id)
        click.echo(dataset)


@click.command(name="columns")
@click.option("--id", required=True, help="Dataset ID")
def dataset_columns(id):
    preferences = Preferences.load()
    click.secho("\n[+] Collecting columns from the dataset\n", fg="blue", bold=True)
    if preferences.current_mode == AppModes.CLOUD:
        dataset = Datasets.get_dataset("cloud", id)
        if dataset is not None:
            click.echo(dataset.column_names)
    else:
        dataset = Datasets.get_dataset("local", id)
        click.echo(dataset)


@click.command(name="download")
@click.option("--id", required=True, help="Dataset ID")
@click.option(
    "--path", required=True, help="Path where the downloaded dataset will be saved"
)
def dataset_download(id, path):
    preferences = Preferences.load()
    click.secho("\n[+] Downloading dataset\n", fg="blue", bold=True)
    if preferences.current_mode == AppModes.CLOUD:
        dataset = Datasets.get_dataset("cloud", id)
        dataset.download(path)


@click.command(name="add")
@click.option(
    "--name", required=True, help="Path where the dataset to upload is located"
)
@click.option(
    "--dir_path", required=True, help="Directory where the dataset to upload is located"
)
@click.option(
    "--type", required=True, help="Dataset type. It should be hf, csv or json"
)
def dataset_add(name, dir_path, type):
    preferences = Preferences.load()

    type = type.strip()
    assert type in ["hf", "csv", "json"], "Dataset type should be hf, csv or json"
    dir_path = Path(dir_path)
    assert dir_path.is_dir(), "The path should be a directory"

    dataset = Dataset(name=name, directory_path=dir_path, dataset_type=type)

    click.secho("\n[+] Uploading dataset...\n", fg="blue", bold=True)
    if preferences.current_mode == AppModes.CLOUD:
        with Spinner():
            dataset.upload(mode="cloud")
    else:
        with Spinner():
            dataset.upload(mode="local")
    click.secho("[+] Dataset uploaded\n", fg="green", bold=True)


@click.command(name="remove")
@click.option("--dataset_id", help="ID of the dataset to be deleted")
def dataset_remove(dataset_id):
    preferences = Preferences.load()

    if preferences.current_mode == AppModes.LOCAL:
        dataset = Datasets.get_dataset("local", dataset_id)
        click.echo("Deleting the following dataset:")
        click.echo(dataset_id)

        click.confirm("Do you want to continue?", abort=True)

        Datasets.remove_local_dataset(dataset_id)
        click.echo("Deleted dataset")
        EventLogger.log_event("datasets_local_remove")


datasets.add_command(ls_datasets)
datasets.add_command(dataset_inspect)
datasets.add_command(dataset_download)
datasets.add_command(dataset_add)
datasets.add_command(dataset_remove)
datasets.add_command(dataset_columns)



from pathlib import Path
from typing_extensions import Required

import click
import requests
import sys
from stochasticx.utils.preferences import Preferences, AppModes
from stochasticx.models.models import Models, Model
from stochasticx.utils.parse_utils import print_table
from stochasticx.constants.urls import LocalRoutes
from stochasticx.utils.auth_utils import AuthUtils
from stochasticx.utils.spinner_slash import Spinner
import sys

from stochasticx.utils.stat_controller import EventLogger

preferences = Preferences.load()


@click.group(name="models")
def models():
    try:
        AuthUtils.get_auth_headers()
    except:
        sys.exit()

    preferences = Preferences.load()

    if preferences.current_mode == AppModes.LOCAL:
        try:
            preferences = Preferences.load()
            response = requests.get(preferences.local_url)
            response.raise_for_status()
        except:
            click.secho(
                "[+] Registry is not initilized. Run the command stochasticx local init",
                fg="red",
                bold=True,
            )
            sys.exit(1)
        EventLogger.log_event("models_local")


@click.command(name="ls")
@click.option("--optimized", is_flag=True, help="Show only optimized models")
@click.option("--all", is_flag=True, help="Show all models")
def ls_models(optimized, all):
    if preferences.current_mode == AppModes.CLOUD:
        if not optimized or all:
            if all:
                click.secho("\n[+] Collecting all models\n", fg="blue", bold=True)
            else:
                click.secho("\n[+] Collecting uploaded models\n", fg="blue", bold=True)

            columns, values = Models.get_models(mode="cloud", fmt="table")
            print_table(columns, values)

        if optimized or all:
            if all:
                click.secho("\n[+] Collecting all models\n", fg="blue", bold=True)
            else:
                click.secho("\n[+] Collecting optimized models\n", fg="blue", bold=True)

            columns, values = Models.get_optimized_models(fmt="table")
            print_table(columns, values)
        EventLogger.log_event("models_cloud_ls")
    else:
        click.secho("\n[+] Collecting all local models\n", fg="blue", bold=True)
        columns, values = Models.get_models(mode="local", fmt="table")
        print_table(columns, values)
        EventLogger.log_event("models_local_ls")


@click.command(name="inspect")
@click.option("--id", required=True, help="Model ID")
def model_inspect(id):
    click.secho("\n[+] Collecting information from this model\n", fg="blue", bold=True)

    if preferences.current_mode == AppModes.CLOUD:
        model = Models.get_model(id, mode="cloud")
        click.echo(model)
    else:
        model = Models.get_model(id, mode="local")
        click.echo(model)


@click.command(name="download")
@click.option("--id", required=True, help="Model ID that you want to download")
@click.option(
    "--path", required=True, help="Path where the downloaded model will be saved"
)
def model_download(id, path):
    click.secho("\n[+] Downloading the model\n", fg="blue", bold=True)

    if preferences.current_mode == AppModes.CLOUD:
        model = Models.get_model(id, mode="cloud")
        model.download(path)
        click.secho("\n[+] Model downloaded\n", fg="green", bold=True)


@click.command(name="add")
@click.option("--name", required=True, help="Path where the model to upload is located")
@click.option(
    "--dir_path", required=True, help="Directory where the model to upload is located"
)
@click.option("--type", required=True, help="Model type. It should be hf, pt or custom")
def model_add(name, dir_path, type):
    type = type.strip()
    assert type in [
        "hf",
        "pt",
        "custom",
        "onnx",
    ], "Model type should be hf, pt or custom"
    dir_path = Path(dir_path)
    assert dir_path.is_dir(), "The path should be a directory"

    model = Model(name=name, directory_path=dir_path, model_type=type)
    click.secho("\n[+] Uploading model...\n", fg="blue", bold=True)
    with Spinner():
        if preferences.current_mode == AppModes.CLOUD:
            model.upload("cloud")
        else:
            model.upload("local")
    click.secho("\n[+] Model uploaded\n", fg="green", bold=True)


@click.command(name="remove")
@click.option("--model_id", help="ID of the model to be deleted")
def model_remove(model_id):
    if preferences.current_mode == AppModes.LOCAL:
        model = Models.get_model(model_id, mode="local")
        click.echo("Deleting the following model:")
        click.echo(model)

        click.confirm("Do you want to continue?", abort=True)

        Models.remove_local_model(model_id)
        click.echo("Deleted model")
        EventLogger.log_event("models_local_remove")


models.add_command(ls_models)
models.add_command(model_inspect)
models.add_command(model_download)
models.add_command(model_add)
models.add_command(model_remove)



import click
from stochasticx.inferences.inference import (
    SequenceClassificationModel,
    QuestionAnsweringModel,
    SummarizationModel,
    TranslationModel,
    TokenClassificationModel,
)

from stochasticx.deployment.deployments import DeploymentsClient

from stochasticx.utils.preferences import Preferences, AppModes
from stochasticx.utils.gpu_utils import is_nvidia_gpu_available
from stochasticx.utils.auth_utils import AuthUtils
from stochasticx.utils.preferences import Preferences
import sys

from stochasticx.utils.stat_controller import EventLogger


@click.group(name="inference")
def inference():
    try:
        AuthUtils.get_auth_headers()
    except:
        sys.exit()
    EventLogger.log_event("inference")


@click.command(name="inspect")
@click.option("--deployment_name", required=True, help="Deployment name")
def inspect_inference(deployment_name):
    deployments = DeploymentsClient.get_deployments(fmt=None)
    
    found_deployment = None
    for deployment in deployments:
        if deployment.name == deployment_name:
            found_deployment = deployment
            break
        
    if found_deployment is None:
        click.secho("\n[+] Deployment {} not found\n".format(deployment_name), fg="red", bold=True)
    elif deployment.status == "deploying":
        click.secho("\n[+] Your deployment {} is still in deploying status. Wait some minutes\n".format(deployment_name), fg="blue", bold=True)
    elif deployment.status == "running":
        click.secho("\n[+] Use these data to start the inference:", fg="green", bold=True)
        click.secho("\t URL: {}".format(deployment.client_url), fg="white", bold=True)
        click.secho("\t API key: {}".format(deployment.api_key), fg="white", bold=True)
    else:
        click.secho("\n[+] You are not able to run inference on this deployment. The status is not deploying / running\n", fg="yellow", bold=True)
        
    EventLogger.log_event("inference_cloud") 
       


@click.command(name="sequence_classification")
@click.option("--deployment_id", required=True, help="Deployment ID")
@click.option("--text", required=True, help="Text")
def sequence_classification(deployment_id, text):
    preferences = Preferences.load()

    inference_model = SequenceClassificationModel(deployment_id)

    if preferences.current_mode == AppModes.LOCAL:
        labels, scores = inference_model.local_inference([text])
        EventLogger.log_event("inference_local_sequence_classification")
    else:
        labels, scores = inference_model.inference([text])
        EventLogger.log_event("inference_cloud_sequence_classification")

    click.echo("Labels: {}".format(labels))
    click.echo("Scores: {}".format(scores))


@click.command(name="question_answering")
@click.option("--deployment_id", required=True, help="Deployment ID")
@click.option("--question", required=True, help="Question")
@click.option("--context", required=True, help="Context")
def question_answering(deployment_id, question, context):
    preferences = Preferences.load()

    if preferences.current_mode == AppModes.LOCAL:
        EventLogger.log_event("inference_local_question_answering")
    else:
        inference_model = QuestionAnsweringModel(deployment_id)

        answer = inference_model.inference([question], [context])
        click.echo("Answer: {}".format(answer))
        EventLogger.log_event("inference_cloud_question_answering")


@click.command(name="summarization")
@click.option("--deployment_id", required=True, help="Deployment ID")
@click.option("--text", required=True, help="Text")
@click.option("--min_length", required=True, help="Min length")
@click.option("--max_length", required=True, help="Max length")
def summarization(deployment_id, text, min_length, max_length):
    preferences = Preferences.load()

    if preferences.current_mode == AppModes.LOCAL:
        EventLogger.log_event("inference_local_summarization")
    else:
        inference_model = SummarizationModel(deployment_id)

        summary = inference_model.inference([text], [min_length], [max_length])
        click.echo("Answer: {}".format(summary))
        EventLogger.log_event("inference_cloud_summarization")


@click.command(name="translation")
@click.option("--deployment_id", required=True, help="Deployment ID")
@click.option("--text", required=True, help="Text")
@click.option("--max_length", required=True, help="Max length")
def translation(deployment_id, text, max_length):
    preferences = Preferences.load()

    if preferences.current_mode == AppModes.LOCAL:
        EventLogger.log_event("inference_local_translation")
    else:
        inference_model = TranslationModel(deployment_id)

        translation = inference_model.inference([text], [max_length])
        click.echo("Answer: {}".format(translation))
        EventLogger.log_event("inference_cloud_translation")


@click.command(name="token_classification")
@click.option("--deployment_id", required=True, help="Deployment ID")
@click.option("--text", required=True, help="Text")
def token_classification(deployment_id, text):
    preferences = Preferences.load()

    if preferences.current_mode == AppModes.LOCAL:
        EventLogger.log_event("inference_local_token_classification")
    else:
        inference_model = TokenClassificationModel(deployment_id)

        tokens, tags, scores = inference_model.inference([text])
        click.echo("Tokens: {}".format(tokens))
        click.echo("Tags: {}".format(tags))
        click.echo("Scores: {}".format(scores))
        EventLogger.log_event("inference_cloud_token_classification")


inference.add_command(inspect_inference)
inference.add_command(sequence_classification)
inference.add_command(question_answering)
inference.add_command(summarization)
inference.add_command(translation)
inference.add_command(token_classification)



class DockerImages:
    BENCHMARK = "public.ecr.aws/t8g5g2q5/x-benchmarking:latest"
    CONVERSION = "public.ecr.aws/t8g5g2q5/x-conversion:latest"
    FINETUNE = "public.ecr.aws/t8g5g2q5/x-finetuning:latest"
    INFERENCE = "public.ecr.aws/t8g5g2q5/x-inference:latest"
    LOCAL = "public.ecr.aws/t8g5g2q5/x-local:latest"


class ContainerNames:
    BENCHMARK = "x-benchmarking"
    CONVERSION = "x-conversion"
    FINETUNE = "x-finetuning"
    INFERENCE = "x-inference"
    LOCAL = "x-local"



from pathlib import Path
import os
import click
import time

from stochasticx.utils.preferences import Preferences, AppModes
from stochasticx.conversion.local_conversion import LocalConversion
from stochasticx.models.models import Models
from stochasticx.utils.docker import (
    start_container,
    exists_container,
    stop_and_remove_container,
    auto_port_selection,
    wait_container_is_up,
)
from stochasticx.constants.docker_images import DockerImages, ContainerNames
from stochasticx.utils.auth_utils import AuthUtils
from stochasticx.constants.urls import get_local_url, LocalRoutes
import sys
import traceback
from stochasticx.utils.stat_controller import EventLogger


@click.group(name="conversion")
def conversion():
    try:
        AuthUtils.get_auth_headers()
    except:
        sys.exit()
    EventLogger.log_event("conversion")


@click.command(name="convert")
@click.option(
    "--port",
    default=5000,
    help="Port which Stochastic local conversion container will run on",
)
@click.option("--model_id", help="Directory where the model to upload is located")
@click.option(
    "--convert_type", help="convert type. It should be onnx, tensorrt or onnx_int8"
)
@click.option("--model_type", help="Model type. It should be hf, pt or custom")
@click.option(
    "--task_type",
    help="Task type. It should be sequence_classification, token_classification, question_answering, summarization,translation",
)
@click.option(
    "--sequence_length",
    help="Sequence length should be specific for tensorrt conversion, not needed for onnx conversion",
)
@click.option(
    "--max_batchsize",
    help="Maximum batch size should be specific for tensorrt conversion, not needed for onnx conversion",
)
def conversion_convert(
    port, model_id, convert_type, model_type, task_type, sequence_length, max_batchsize
):
    preferences = Preferences.load()
    if (
        not exists_container(ContainerNames.CONVERSION)
        and preferences.current_mode == AppModes.LOCAL
    ):
        click.secho(
            "\n[+] We are configuring the environment for model conversion. If it is the first time, it will take some minutes\n",
            fg="yellow",
            bold=True,
        )

        original_port = port
        port, is_replace = auto_port_selection(port, ContainerNames.CONVERSION)
        if is_replace:
            click.secho(
                f"[+] Port {original_port} is using on another service, port {port} will be selected to run stochasticx model conversion",
                fg="blue",
                bold=True,
            )

        start_container(
            docker_image=DockerImages.CONVERSION,
            ports={
                "5000": port,
            },
            container_name=ContainerNames.CONVERSION,
            detach=True,
            gpu=True,
            volumes=["stochasticx:/vol"],
        )

        click.echo("[+] Setting up preferences...")
        preferences = Preferences.load()
        preferences.current_mode = AppModes.LOCAL
        preferences.local_conversion_url = f"http://127.0.0.1:{port}"
        Preferences.save(preferences)

        local_url = get_local_url(
            LocalRoutes.ONNX_CONVERSION_URL, "local_conversion_url"
        )

        wait_container_is_up(local_url)

        click.secho(
            f"[+] x-local-conversion server running in the port {port}",
            fg="green",
            bold=True,
        )

    convert_type = convert_type.strip()
    assert convert_type in [
        "onnx",
        "tensorrt",
        "onnx_int8",
    ], "Conversion type should be onnx, tensorrt, onnx_int8"
    assert task_type in [
        "sequence_classification",
        "token_classification",
        "question_answering",
        "summarization",
        "translation",
    ], "Task type should be sequence_classification, token_classification, question_answering, summarization,translation"
    assert model_type in ["hf", "pt"], "Model type should be hf or pt"
    model = Models.get_model(model_id, "local")
    model_path = Path(model.directory_path)  # Path(path)
    if convert_type == "tensorrt" or convert_type == "onnx_int8":
        model_path = os.path.join(model_path, "model_onnx.onnx")
        # assert os.path.exists(model_path), "Can not find model_onnx.onnx in your provided directory"

    convert_params = {}
    if sequence_length is not None:
        convert_params["sequence_length"] = sequence_length
    if max_batchsize is not None:
        convert_params["max_batch_size"] = max_batchsize

    if preferences.current_mode == AppModes.LOCAL:
        model = LocalConversion(
            model_path=model_path,
            convert_type=convert_type,
            model_type=model_type,
            task_type=task_type,
            convert_params=convert_params,
        )
        click.secho("\n[+] Converting model ...\n", fg="blue", bold=True)

        result = model.convert()
        if result["status"] == "successful":
            click.secho("\n[+] Model is converted\n", fg="green", bold=True)
        else:
            click.secho(
                f"\n[-] Convertion has failed, the error is:\n {str(result['message'])}",
                fg="red",
                bold=True,
            )

        if exists_container(ContainerNames.CONVERSION):
            click.secho(
                f"[+] Stopping and removing {ContainerNames.CONVERSION}",
                fg="blue",
                bold=True,
            )
            stop_and_remove_container(ContainerNames.CONVERSION)
            click.secho("[+] Removed", fg="green", bold=True)
    EventLogger.log_event("conversion_convert")


conversion.add_command(conversion_convert)



from dataclasses import dataclass, field, asdict
import requests
from datetime import datetime
import click
import sys
from typing import Dict
import uuid
from stochasticx.constants.urls import CloudRoutes, get_cloud_url
from stochasticx.utils.auth_utils import AuthUtils
from stochasticx.models.models import OptimizedModel, Models
from stochasticx.utils.logging import configure_logger
from stochasticx.utils.parse_utils import parse_date
from stochasticx.instances.instance_types import InstanceType
from stochasticx.utils.stat_controller import (
    StatController,
    LocalDeploymentJobInformation,
)
from stochasticx.utils.preferences import Preferences
from stochasticx.constants.urls import (
    DEPLOYMENTS_URL, 
    STOCHASTIC_TEMPLATES_URL,
    CUSTOM_TEMPLATES_URL,
    IS_INFERENCE_FILE_AVAILABLE_URL
)

import requests
from dataclasses import dataclass

from stochasticx.models.models import Model
from stochasticx.utils.auth_utils import AuthUtils


logger = configure_logger(__name__)


class LocalDeploymentsClient:
    @staticmethod
    def get_deployments(fmt="table"):
        preferences = Preferences.load()
        response = requests.get(f"{preferences.local_deployment_url}/models")
        response.raise_for_status()

        models = response.json()
        model_results = []

        for model in models:
            model_results.append(
                LocalDeployment(
                    id=model.get("id"),
                    model_name=model.get("model_name"),
                    type=model.get("type"),
                    cuda=model.get("cuda"),
                    task_type=model.get("task_type"),
                    user_params=model.get("user_params"),
                )
            )

        if fmt == "table":
            columns = []
            values = []

            if len(model_results) > 0:
                columns, _ = model_results[0].to_table()

            for model in model_results:
                _, vals = model.to_table()
                values.append(vals)

            return columns, values
        else:
            return model_results

    @staticmethod
    def get_deployment_by_id(id):
        deployments = LocalDeploymentsClient.get_deployments(fmt=None)

        for deployment in deployments:
            if deployment.id == id:
                return deployment

        return None

    @staticmethod
    def delete_deployment(model_type, model_name):
        preferences = Preferences.load()
        response = requests.delete(
            "{}/model/{}/{}".format(
                preferences.local_deployment_url, model_type, model_name
            )
        )
        response.raise_for_status()


@dataclass
class LocalDeployment:
    model_name: str
    type: str
    cuda: bool
    task_type: str
    user_params: Dict
    id: str = None

    def to_table(self):
        columns = [
            "Model ID",
            "Model name",
            "Model type",
            "CUDA",
            "Task type",
            "User params",
        ]

        values = [
            str(self.id),
            self.model_name,
            self.type,
            str(self.cuda),
            self.task_type,
            str(self.user_params),
        ]

        return columns, values


@dataclass
class LocalSequenceClassificationDeployment(LocalDeployment):
    task_type: str = "sequence_classification"
    user_params: Dict = field(
        default_factory=lambda: {"max_batch_size": 8, "max_seq_length": 128}
    )

    def start_deployment(self):
        preferences = Preferences.load()
        response = requests.post(
            f"{preferences.local_deployment_url}/model", json=asdict(self)
        )

        response.raise_for_status()

        StatController.add_information(
            LocalDeploymentJobInformation(
                deployment_type=self.task_type,
                model_name=self.model_name,
                type=self.type,
                params=self.user_params,
                job_result=response.json(),
            )
        )


@dataclass
class LocalQuestionAnsweringDeployment(LocalDeployment):
    task_type: str = "question_answering"
    user_params: Dict = field(
        default_factory=lambda: {
            "max_batch_size": 8,
            "max_seq_length": 256,
            "stride": 30,
        }
    )

    def start_deployment(self):
        preferences = Preferences.load()
        response = requests.post(
            f"{preferences.local_deployment_url}/model", json=asdict(self)
        )

        response.raise_for_status()

        StatController.add_information(
            LocalDeploymentJobInformation(
                deployment_type=self.task_type,
                model_name=self.model_name,
                type=self.type,
                params=self.user_params,
                job_result=response.json(),
            )
        )


@dataclass
class LocalTokenClassificationDeployment(LocalDeployment):
    task_type: str = "token_classification"
    user_params: Dict = field(
        default_factory=lambda: {"max_seq_length": 256, "label_all_tokens": False}
    )

    def start_deployment(self):
        preferences = Preferences.load()
        response = requests.post(
            f"{preferences.local_deployment_url}", json=asdict(self)
        )

        response.raise_for_status()

        StatController.add_information(
            LocalDeploymentJobInformation(
                deployment_type=self.task_type,
                model_name=self.model_name,
                type=self.type,
                params=self.user_params,
                job_result=response.json(),
            )
        )


@dataclass
class LocalTranslationDeployment(LocalDeployment):
    task_type: str = "translation"
    user_params: Dict = field(
        default_factory=lambda: {
            "max_source_length": 256,
            "max_target_length": 256,
            "src_lang": "en",
            "tgt_lang": "es",
            "pad_to_max_length": False,
            "num_beams": 4,
            "ignore_pad_token_for_loss": True,
            "source_prefix": "",
            "forced_bos_token": None,
        }
    )

    def start_deployment(self):
        preferences = Preferences.load()
        response = requests.post(
            f"{preferences.local_deployment_url}/model", json=asdict(self)
        )

        response.raise_for_status()

        StatController.add_information(
            LocalDeploymentJobInformation(
                deployment_type=self.task_type,
                model_name=self.model_name,
                type=self.type,
                params=self.user_params,
                job_result=response.json(),
            )
        )


@dataclass
class LocalSummarizationDeployment(LocalDeployment):
    task_type: str = "summarization"
    user_params: Dict = field(
        default_factory=lambda: {
            "max_source_length": 256,
            "max_target_length": 64,
            "lang": "en",
            "pad_to_max_length": False,
            "num_beams": 4,
            "ignore_pad_token_for_loss": True,
            "source_prefix": "",
            "forced_bos_token": None,
        }
    )

    def start_deployment(self):
        preferences = Preferences.load()
        response = requests.post(
            f"{preferences.local_deployment_url}/model", json=asdict(self)
        )

        response.raise_for_status()

        StatController.add_information(
            LocalDeploymentJobInformation(
                deployment_type=self.task_type,
                model_name=self.model_name,
                type=self.type,
                params=self.user_params,
                job_result=response.json(),
            )
        )


@dataclass
class LocalTextGenerationDeployment(LocalDeployment):
    task_type: str = "text_generation"
    user_params: Dict = field(default_factory=lambda: {"block_size": 256})

    def start_deployment(self):
        preferences = Preferences.load()
        response = requests.post(
            f"{preferences.local_deployment_url}/model", json=asdict(self)
        )

        response.raise_for_status()

        StatController.add_information(
            LocalDeploymentJobInformation(
                deployment_type=self.task_type,
                model_name=self.model_name,
                type=self.type,
                params=self.user_params,
                job_result=response.json(),
            )
        )


class InstanceTypes:
    g4dn_xlarge = "g4dn.xlarge"
    c5_2xlarge = "c5.2xlarge"
    c5_12xlarge = "c5.12xlarge"


class Instance:
    def __init__(
        self,
        id,
        name,
        cost_per_hour,
        cost_per_month,
        spot_cost,
        storage,
        vcpus,
        memory,
        network,
    ):
        self.id = id
        self.name = name
        self.cost_per_hour = cost_per_hour
        self.cost_per_month = cost_per_month
        self.spot_cost = spot_cost
        self.storage = storage
        self.vcpus = vcpus
        self.memory = memory
        self.network = network

    def to_table(self):
        columns = [
            "Id",
            "Name",
            "Cost/hour",
            "Cost/month",
            "Spot cost",
            "Storage",
            "vCPUs",
            "Memory",
            "Network",
        ]

        values = [
            self.id,
            self.name,
            self.cost_per_hour,
            self.cost_per_month,
            self.spot_cost,
            self.storage,
            self.vcpus,
            self.memory,
            self.network,
        ]

        return columns, values

    def __str__(self):
        return "ID: {} ; Name: {} ; Cost/hour: {} ; Cost/month: {} ; Spot cost: {} ; Storage: {} ; vCPUs: {} ; Memory: {} ; Network: {}".format(
            self.id,
            self.name,
            self.cost_per_hour,
            self.cost_per_month,
            self.spot_cost,
            self.storage,
            self.vcpus,
            self.memory,
            self.network,
        )


class Instances:
    @staticmethod
    def get_instance_types(fmt=None):
        instances = []

        auth_header = AuthUtils.get_auth_headers()
        r = requests.get(get_cloud_url(CloudRoutes.INSTANCES_URL), headers=auth_header)
        r.raise_for_status()

        data = r.json()

        if data.get("ec2Instances") is not None:
            for instance_data in data["ec2Instances"]:
                instance = Instance(
                    id=instance_data["id"],
                    name=instance_data["instanceType"],
                    cost_per_hour=instance_data["hourlyCost"],
                    memory=instance_data["memory"],
                    cost_per_month=instance_data["monthlyCost"],
                    network=instance_data["network"],
                    spot_cost=instance_data["spotCost"],
                    storage=instance_data["storage"],
                    vcpus=instance_data["vcpus"],
                )

                instances.append(instance)

            if fmt == "table":
                columns = []
                values = []

                if len(instances) > 0:
                    columns, _ = instances[0].to_table()

                for instance in instances:
                    _, vals = instance.to_table()
                    values.append(vals)

                return columns, values

        return instances

    @staticmethod
    def get_instance_type_by_name(name):
        instances = Instances.get_instance_types()

        for instance in instances:
            if instance.name == name:
                return instance


class StableDiffusionDeployment:
    def __init__(self, id, status, model_name, api_key=None, client_url=None):
        self.id = id
        self.status = status
        self.model_name = model_name
        self.api_key = api_key
        self.client_url = client_url

    def __str__(self):
        return (
            "ID: {} ; Status: {} ; Model name: {} ; API key: {} ; Endpoint: {}".format(
                self.id, self.status, self.model_name, self.api_key, self.client_url
            )
        )
    
     
class StableDiffusionDeployments:
    @staticmethod
    def get_deployments():
        deployments = []

        auth_header = AuthUtils.get_auth_headers()
        r = requests.get(
            get_cloud_url(CloudRoutes.STABLE_DIFFUSION_URL), headers=auth_header
        )
        r.raise_for_status()

        data = r.json()
        deploying = False

        if data["data"] is not None:
            for deployment_data in data["data"]:

                api_key = None
                client_url = None
                if deployment_data.get("resources") is not None:
                    api_key = deployment_data.get("resources").get("apiKey")
                    client_url = deployment_data.get("clientUrl")
                else:
                    deploying = True

                deployment = StableDiffusionDeployment(
                    id=deployment_data.get("id"),
                    status=deployment_data.get("status"),
                    model_name=deployment_data.get("modelName"),
                    client_url=client_url,
                    api_key=api_key,
                )
                deployments.append(deployment)

        if deploying:
            click.secho(
                "[+] There are models that are still deploying. It can take some minutes...\n",
                bold=True,
                fg="yellow",
            )

        return deployments

    @staticmethod
    def get_deployment(id):
        url = get_cloud_url(CloudRoutes.STABLE_DIFFUSION_URL.formating(id))

        auth_header = AuthUtils.get_auth_headers()
        r = requests.get(url, headers=auth_header)

        try:
            r.raise_for_status()
        except:
            click.secho("[+] The ID is not correct\n", bold=True, fg="yellow")
            sys.exit()

        data = r.json()
        deployment_data = data["deployedModel"]
        if deployment_data is not None:
            if deployment_data.get("resources") is not None:
                api_key = deployment_data.get("resources").get("apiKey")
                client_url = deployment_data.get("clientUrl")

                deployment = StableDiffusionDeployment(
                    id=deployment_data.get("_id"),
                    status=deployment_data.get("status"),
                    model_name=deployment_data.get("modelName"),
                    client_url=client_url,
                    api_key=api_key,
                )
            else:
                deployment = StableDiffusionDeployment(
                    id=deployment_data.get("_id"),
                    status=deployment_data.get("status"),
                    model_name=deployment_data.get("modelName"),
                )

                click.secho(
                    "[+] The model is still deploying. It can take some minutes...",
                    bold=True,
                    color="yellow",
                )

            return deployment

        return None

    @staticmethod
    def deploy():
        auth_header = AuthUtils.get_auth_headers()
        r = requests.post(
            get_cloud_url(CloudRoutes.STABLE_DIFFUSION_URL),
            headers=auth_header,
            json={"name": "Stable-Diffusion-{}".format(uuid.uuid4())},
        )
        r.raise_for_status()

    @staticmethod
    def delete(id):
        url = get_cloud_url(CloudRoutes.STABLE_DIFFUSION_URL.formating(id))
        auth_header = AuthUtils.get_auth_headers()
        r = requests.delete(url, headers=auth_header)
        r.raise_for_status()


@dataclass
class Deployment:
    id: str
    name: str
    status: str
    instance_type: str
    client_url: str
    api_key: str
    model: Model
    inference_file_path: str = None
    
    def to_table(self):
        columns = [
            "Name", 
            "Status",
            "Instance",
            "Model"
        ]
        
        values = [
            self.name,
            self.status,
            self.instance_type,
            self.model.name
        ]
        
        return columns, values
    
    def __str__(self) -> str:
        return "Name {} ; Status {} ; Model {}".format(
            self.name,
            self.status,
            self.model.name
        )


class DeploymentsClient:
    @staticmethod
    def get_deployments(fmt="table"):
        deployments = []
        auth_header = AuthUtils.get_auth_headers()
        response = requests.get(
            url=DEPLOYMENTS_URL,
            headers=auth_header
        )
        
        response.raise_for_status()
        
        deployments_data_list = response.json()["data"]
                
        for deployment_data in deployments_data_list:
            try:
                deployment = Deployment(
                    id = deployment_data["id"],
                    name = deployment_data["name"],
                    client_url= deployment_data.get("clientUrl") if deployment_data.get("clientUrl") else "Not available",
                    instance_type = deployment_data.get("instanceType"),
                    status = deployment_data.get("status"),
                    api_key = deployment_data["resources"]["apiKey"] if deployment_data.get("resources") is not None else "Not available",
                    model=Model(
                        name=deployment_data["model"]["name"],
                        directory_path=None,
                        model_type=deployment_data["model"]["type"]
                    ) if deployment_data.get("model") is not None else "Not available"
                )
                
                deployments.append(deployment)
            except Exception as ex:
                pass
            
        if fmt == "table":
            columns = []
            values = []
            
            if len(deployments) > 0:
                columns, _ = deployments[0].to_table()
                
            for deployment in deployments:
                _, vals = deployment.to_table()
                values.append(vals)
                
            return columns, values
        
        return deployments
        
    @staticmethod
    def get_deployment_by_name(name):
        auth_header = AuthUtils.get_auth_headers()
        
        deployments = DeploymentsClient.get_deployments(fmt=None)
        
        deployment_id = None
        for deployment in deployments:
            if deployment.name == name:
                deployment_id = deployment.id
                break
                
        if deployment_id is None:
            raise ValueError(f"The deployment {name} does not exist")
        
        response = requests.get(
            url=f"{DEPLOYMENTS_URL}/{deployment_id}",
            headers=auth_header
        )
        
        response.raise_for_status()
        
        deployment_data = response.json()["data"]
                
        deployment = Deployment(
            id = deployment_data["id"],
            name = deployment_data["name"],
            client_url= deployment_data.get("clientUrl") if deployment_data.get("clientUrl") else "Not available",
            instance_type = deployment_data.get("instanceType"),
            status = deployment_data.get("status"),
            api_key = deployment_data["resources"]["apiKey"] if deployment_data.get("resources") is not None else "Not available",
            model=Model(
                name=deployment_data["model"]["name"],
                directory_path=None,
                model_type=deployment_data["model"]["type"]
            ) if deployment_data.get("model") is not None else "Not available"
        )
        
        return deployment
        
    @staticmethod
    def create_deployment(
        name,
        instance_type,
        model_name,
        model_type=None,
        task_type=None,
        inference_file_path=None,
        docker_image=None,
        docker_registry_username=None,
        docker_registry_password=None
    ):        
        valid_instances = [InstanceType.G4DN_XLARGE]
        assert instance_type in valid_instances, "Your instance is not valid. Valid instances: {}".format(valid_instances)
        
        auth_header = AuthUtils.get_auth_headers()
        
        models = Models.get_models(mode="cloud", fmt=None)
        model_id = None
        if len(models) > 0:
            
            for model in models:
                if model.name == model_name:
                    model_id = model.model_id
                    break
                
        if model_id is None:
            logger.error("The model name {} was not found".format(model_name))
            sys.exit()
        
        if model_type is not None and task_type is not None:
            response = requests.post(
                url=f"{STOCHASTIC_TEMPLATES_URL}",
                headers=auth_header,
                json={
                    "model_id": model_id,
                    "model_type": model_type,
                    "task_type": task_type
                }
            )
            
            response.raise_for_status()
        
        if inference_file_path is not None:
            response = requests.post(
                url=f"{CUSTOM_TEMPLATES_URL}",
                headers=auth_header,
                json={
                    "model_id": model_id,
                    "inference_path": inference_file_path
                }
            )
            
            response.raise_for_status()
        
        # Check inference file is available in the model
        response = requests.post(
            url=f"{IS_INFERENCE_FILE_AVAILABLE_URL}",
            headers=auth_header,
            json={
                "model_id": model_id
            }
        )
        
        response.raise_for_status()
        is_inference_file_available = response.json()["data"]
        
        if not is_inference_file_available:
            raise ValueError("There is not inference.py file in the model you have specified")
        
        response = requests.post(
            url=f"{DEPLOYMENTS_URL}",
            headers=auth_header,
            json={
                "name": name,
                "model_id": model_id,
                "instance_type": instance_type,
                #"docker_image": docker_image,
                #"docker_registry_username": docker_registry_username,
                #"docker_registry_password": docker_registry_password
            }
        )
        
        response.raise_for_status()
    
    @staticmethod
    def delete_deployment(deployment_name):
        auth_header = AuthUtils.get_auth_headers()
        
        deployments = DeploymentsClient.get_deployments(fmt=None)
        
        deployment_id = None
        for deployment in deployments:
            if deployment.name == deployment_name:
                deployment_id = deployment.id
                break
                
        if deployment_id is None:
            logger.error("The deployment name {} was not found".format(deployment_name))
            return
        
        response = requests.delete(
            url=f"{DEPLOYMENTS_URL}/{deployment_id}",
            headers=auth_header
        )
        
        response.raise_for_status()
    
    @staticmethod
    def stop_deployment(deployment_id):
        auth_header = AuthUtils.get_auth_headers()
        
        response = requests.put(
            url=f"{DEPLOYMENTS_URL}/{deployment_id}/stop",
            headers=auth_header
        )
        
        response.raise_for_status()
    
    @staticmethod
    def resume_deployment(deployment_id):
        auth_header = AuthUtils.get_auth_headers()
        
        response = requests.put(
            url=f"{DEPLOYMENTS_URL}/{deployment_id}/start",
            headers=auth_header
        )
        
        response.raise_for_status()      



import os
import json
import time

import requests
import sys
import click
from pathlib import Path
import uuid

from stochasticx.constants.urls import CloudRoutes, TOKEN_AUTH_PATH, get_cloud_url

WAIT_TOKEN_TIME = 60
BEFORE_RETRY = 2


class OAuth2Links:
    user_link = "https://api.stochastic.ai/v1/auth/google?clientId={}"
    token_link = "https://api.stochastic.ai/v1/auth/cli-login?clientId={}"


class LoginUtils:
    @staticmethod
    def login_request(username, password):
        """Private method to do the login request

        Args:
            username (str): username
            password (str): password

        Returns:
            str: token
        """
        login_info = {"email": username, "password": password}

        response = requests.post(get_cloud_url(CloudRoutes.LOGIN_URL), data=login_info)
        try:
            response.raise_for_status()
        except:
            click.secho("[+] Your credentials are incorrect.\n", fg="red", bold=True)
            sys.exit()

        response_data = response.json()
        token = response_data["token"]

        LoginUtils.save_token(token)

        return token

    @staticmethod
    def link_login_request():
        uuid_string = str(uuid.uuid4())

        click.secho(OAuth2Links.user_link.format(uuid_string), fg="blue")

        click.secho("Receiving your token...", fg="green")

        token = None

        def not_successful():
            click.secho("[+] Failed to get token, try again.\n", fg="red", bold=True)
            sys.exit()

        for _ in range(WAIT_TOKEN_TIME // BEFORE_RETRY):
            try:
                response = requests.get(OAuth2Links.token_link.format(uuid_string))

                if response.status_code == 200:
                    token_json = response.json()

                    if len(token_json) == 0:
                        time.sleep(BEFORE_RETRY)
                        continue

                    token = token_json["token"]

                    break

            except Exception as e:
                not_successful()

        if token is None:
            not_successful()

        # here query for /user/me to get the email (username), token in header
        response = requests.get(
            get_cloud_url(CloudRoutes.ME_URL),
            headers={
                'Authorization': 'Bearer ' + token
            }
        )
        os.environ["STOCHASTIC_USER"] = response.json().get("email")
        LoginUtils.save_token(token)

        return token

    @staticmethod
    def save_token(token):
        # Save token in ENV var
        session = str(uuid.uuid4())
        os.environ["SESSION_STRING"] = session
        os.environ["STOCHASTIC_TOKEN"] = token
        # Save token in a file
        token_path = Path(TOKEN_AUTH_PATH).resolve()
        if not token_path.exists():
            token_path.parent.mkdir(parents=True, exist_ok=True)
        with open(str(token_path), "w", encoding="utf-8") as f:
            json.dump(
                {
                    "token": token,
                    "session": session,
                    "user": os.environ.get("STOCHASTIC_USER"),
                },
                f,
                ensure_ascii=False,
                indent=4,
            )


class AuthUtils:
    @staticmethod
    def get_auth_headers():
        """Get authentication headers from the webapp

        Raises:
            ValueError: if token not found in the following ENV variable STOCHASTIC_TOKEN

        Returns:
            dict: auth headers
        """
        token = os.getenv("STOCHASTIC_TOKEN")
        session_string = os.getenv("SESSION_STRING")

        if token is None or session_string is None:
            if Path(TOKEN_AUTH_PATH).exists():
                f = open(TOKEN_AUTH_PATH)
                data = json.load(f)
                token = data["token"]
                session_string = data.get("session")
                #os.environ["STOCHASTIC_USER"] = data.get("user")

        os.environ["SESSION_STRING"] = session_string

        if token is not None:
            try:
                response = requests.get(
                    get_cloud_url(CloudRoutes.ME_URL),
                    headers={"Authorization": "Bearer " + token},
                )
                response.raise_for_status()
            except:
                click.secho(
                    "[+] You are not logged in. Please execute:", fg="red", bold=True
                )
                click.secho("\n     stochasticx login \n", bold=True)

                sys.exit()

        else:
            click.secho("[+] You are not logged in. Please try:", fg="red", bold=True)
            click.secho("\n     stochasticx login \n", bold=True)
            click.secho(
                "[+] You can sign up for a free account at: https://app.stochastic.ai/signup"
            )

            sys.exit()

        return {"Authorization": "Bearer " + token}


    @staticmethod
    def is_logged_in():
        """Check if user is logged in

        Returns:
            bool: True if user is logged in, False otherwise
        """
        token_path = Path(TOKEN_AUTH_PATH).resolve()
        return token_path.exists()



from stochasticx.constants.urls import (
    LocalRoutes,
    get_local_url,
    CloudRoutes,
    get_cloud_url,
)
from stochasticx.utils.auth_utils import AuthUtils
from stochasticx.utils.file_utils import ModelUtils
import requests

from stochasticx.utils.stat_controller import EventLogger


class ModelType:
    """Model type"""

    HUGGINGFACE = "hf"
    PYTORCH = "pt"
    ONNX = "onnx"
    CUSTOM = "custom"


class Model:
    """Model class"""

    def __init__(
        self, name: str, directory_path: str, model_type: str = ModelType.HUGGINGFACE
    ):
        """Initializer

        Args:
            name (str): model name
            directory_path (str): directory path where the model is located
            model_type (str, optional): model type. Defaults to ModelType.HUGGINGFACE.
        """
        assert isinstance(name, str), "The provided name {} is not valid".format(name)

        self.name = name
        self.directory_path = directory_path
        self.model_type = model_type
        self.model_id = None
        self.model_info = None
        self.is_uploaded = False

    def upload(self, mode="cloud"):
        """Upload the model to the stochastic platform"""

        model_id = ModelUtils.upload_model(
            self.directory_path, self.name, self.model_type, mode
        )

        EventLogger.log_event(f"models_{mode}_upload")

        self.set_id(model_id)
        self.is_uploaded = True
        
    def delete(self):
        ModelUtils.de

    def set_id(self, model_id: str):
        """Sets the model ID

        Args:
            model_id (str): new ID
        """
        self.model_id = model_id

    def sync(self):
        """Syncs the model information with the stochastic platform"""

        if self.model_id is not None:
            temp_model = Models.get_model(self.model_id, "cloud")
            self.name = temp_model.name
            self.model_info = temp_model.model_info
            self.is_uploaded = True

    def get_id(self):
        """Gets the model ID

        Returns:
            str: the model ID
        """
        return self.model_id

    def get_model_info(self):
        """Gets the model information

        Returns:
            dict: model information
        """

        self.sync()

        return self.model_info

    def set_model_info(self, model_info):
        """Sets the model information

        Args:
            model_info (dict): model info
        """
        self.model_info = model_info

    def download(self, local_path: str):
        """Download the model for the Stochastic platform

        Args:
            local_path (str): path where the model will be saved
        """
        assert self.model_id is not None
        ModelUtils.download_model(self.model_id, local_path)

    def to_table(self):
        columns = ["Id", "Name", "Directory path", "Type", "Uploaded"]
        values = [
            str(self.model_id),
            self.name,
            self.directory_path,
            self.model_type,
            str(self.is_uploaded),
        ]

        return columns, values

    def __str__(self):
        """Convert the object to string

        Returns:
            str
        """
        return "Model ID: {} ; Name: {} ; Directory path: {} ; Model type: {} ; Uploaded: {}".format(
            self.model_id,
            self.name,
            self.directory_path,
            self.model_type,
            self.is_uploaded,
        )


class OptimizedModel:
    """Optimized model class"""

    def __init__(
        self, id: str, name: str, type: str, size: str, benchmark_results: dict
    ):
        """Initializer

        Args:
            id (str): model ID
            name (str): model name
            type (str): model type
            size (str): model size in bytes
            benchmark_results (dict): benchmark results
        """

        self.id = id
        self.name = name
        self.type = type
        self.size = size
        self.benchmark_results = benchmark_results

    def download(self, local_path: str):
        """Downloads the model from the Stochastic platform

        Args:
            local_path (str): path where the model will be saved
        """

        ModelUtils.download_optimized_model(self.id, local_path)

    def get_benchmark_results(self):
        """Get benchmark results

        Returns:
            dict: benchmark results
        """
        return self.benchmark_results

    def to_table(self):
        columns = ["Id", "Name", "Type", "Size (MB)"]
        values = [self.id, self.name, self.type, str(self.size)]

        return columns, values

    def __str__(self):
        """Converts the object to a string

        Returns:
            str
        """

        return "Optimized model ID: {} ; Name: {} ; Type: {} ; Size: {} ".format(
            self.id, self.name, self.type, self.size
        )


class Models:
    """Class to get the models uploaded in the Stochastic platform"""

    @staticmethod
    def get_model(model_id: str, mode: str):
        """Gets the model from the Stochastic platform

        Args:
            model_id (str): model ID

        Returns:
            Model: model
        """
        if mode == "cloud":
            suffix = CloudRoutes.MODELS_URL.formatting(model_id)
            url = get_cloud_url(suffix)
            auth_header = AuthUtils.get_auth_headers()
        else:
            auth_header = None
            url = get_local_url(LocalRoutes.MODEL_URL, "local_url") + "/{}".format(
                model_id
            )

        response = requests.get(url, headers=auth_header)
        model_data = response.json().get("data")

        EventLogger.log_event(f"models_{mode}_get")

        if model_data is not None:
            model = Model(
                name=model_data.get("name"),
                directory_path=None if mode == "cloud" else model_data.get("path"),
                model_type=model_data.get("type"),
            )

            model.set_id(model_data.get("id"))
            if mode == "cloud":
                model.set_model_info(model_data.get("modelInfo"))
            model.is_uploaded = True

            return model

        return None
    
    @staticmethod
    def delete_model(model_name: str, mode: str = "cloud"):
        models = Models.get_models(mode=mode)
        model_id = None
        
        for model in models:
            if model.name == model_name:
                model_id = model.model_id
         
        if model_id is None:     
            raise ValueError("The model {} doesn't exist".format(model_name))
        
        suffix = CloudRoutes.MODELS_URL.formatting(model_id)
        url = get_cloud_url(suffix)
        auth_header = AuthUtils.get_auth_headers()
        
        response = None
        try:
            response = requests.delete(url, headers=auth_header)
            response.raise_for_status()
        except:
            raise ValueError(response.text)

    @staticmethod
    def get_local_model(model_id: str):
        """Gets the model from the Stochastic platform

        Args:
            model_id (str): model ID

        Returns:
            Model: model
        """

        url = get_local_url(LocalRoutes.MODEL_URL, "local_url") + "/{}".format(model_id)

        auth_header = None
        response = requests.get(url, headers=auth_header)
        model_data = response.json().get("data")

        if model_data is not None:
            model = Model(
                name=model_data.get("name"),
                directory_path=model_data.get("path"),
                model_type=model_data.get("type"),
            )

            model.set_id(model_data.get("id"))
            # model.set_model_info(model_data.get("modelInfo"))
            model.is_uploaded = True

            return model

        return None

    @staticmethod
    def get_models(mode, fmt=None):
        """Gets all the models from the stochastic platform

        Returns:
            List[Model]: list of models
        """
        models = []

        if mode == "cloud":
            auth_header = AuthUtils.get_auth_headers()
            url = get_cloud_url(CloudRoutes.MODELS_URL)
        else:
            auth_header = None
            url = get_local_url(LocalRoutes.MODEL_URL, "local_url")
            
        response = requests.get(url, headers=auth_header)
        models_data = response.json().get("data")

        if models_data is not None:
            for model_data in models_data:
                model = Model(
                    name=model_data.get("name"),
                    directory_path=None if mode == "cloud" else model_data.get("path"),
                    model_type=model_data.get("type"),
                )

                model.set_id(model_data.get("id"))
                if mode == "cloud":
                    model.set_model_info(model_data.get("modelInfo"))
                model.is_uploaded = True
                models.append(model)

            if fmt == "table":
                columns = []
                values = []

                if len(models) > 0:
                    columns, _ = models[0].to_table()

                for model in models:
                    _, vals = model.to_table()
                    values.append(vals)

                return columns, values

        return models

    @staticmethod
    def get_local_models(fmt="table"):
        """Gets all the models from the local registry

        Returns:
            List[Model]: list of models
        """
        models = []

        # auth_header = AuthUtils.get_auth_headers()
        auth_header = None
        response = requests.get(
            get_local_url(LocalRoutes.MODEL_URL, "local_url"), headers=auth_header
        )
        models_data = response.json().get("data")

        if models_data is not None:
            for model_data in models_data:

                model = Model(
                    name=model_data.get("name"),
                    directory_path=model_data.get("path"),
                    model_type=model_data.get("type"),
                )

                model.set_id(model_data.get("id"))
                # model.set_model_info(model_data.get("modelInfo"))
                model.is_uploaded = True
                models.append(model)

            if fmt == "table":
                columns = []
                values = []

                if len(models) > 0:
                    columns, _ = models[0].to_table()

                for model in models:
                    _, vals = model.to_table()
                    values.append(vals)

                return columns, values

        return models

    @staticmethod
    def get_optimized_models(fmt="table"):
        """Gets all the optimized models from the Stochastic platform

        Returns:
            List[OptimizedModel]: list of optimized models
        """

        models = []

        auth_header = AuthUtils.get_auth_headers()

        suffix = CloudRoutes.OPTIMIZED_MODELS_URL
        url = get_cloud_url(suffix)

        response = requests.get(url, headers=auth_header)
        models_data = response.json().get("data")

        if models_data is not None:
            for model_data in models_data:
                model = OptimizedModel(
                    id=model_data.get("id"),
                    name=model_data.get("userGivenName"),
                    type=model_data.get("type"),
                    size=model_data.get("size"),
                    benchmark_results=model_data.get("result").get("results"),
                )

                models.append(model)

            if fmt == "table":
                columns = []
                values = []

                if len(models) > 0:
                    columns, _ = models[0].to_table()

                for model in models:
                    _, vals = model.to_table()
                    values.append(vals)

                return columns, values

        if fmt == "table":
            return [], []

        return models

    @staticmethod
    def get_optimized_model(model_id: str):
        """Gets an optimized model from the Stochastic plarform

        Args:
            model_id (str): model ID

        Returns:
            OptimizedModel: optimized model
        """

        suffix = CloudRoutes.OPTIMIZED_MODELS_URL.formatting(model_id)
        url = get_cloud_url(suffix)

        auth_header = AuthUtils.get_auth_headers()
        response = requests.get(url, headers=auth_header)
        model_data = response.json().get("data")

        if model_data is not None and len(model_data) > 0:
            model = OptimizedModel(
                id=model_data.get("id"),
                name=model_data.get("userGivenName"),
                type=model_data.get("type"),
                size=model_data.get("size"),
                benchmark_results=model_data.get("result").get("results"),
            )

            return model

        return None

    @staticmethod
    def remove_local_model(model_id: str):
        """Removes the local model

        Args:
            model_id (str): model ID

        Returns:
            None
        """

        url = get_local_url(LocalRoutes.MODEL_URL, "local_url") + "/{}".format(model_id)

        auth_header = None
        response = requests.delete(url, headers=auth_header)
        status = response.json().get("data")

        return status



import sys
from unicodedata import name
import docker
import time
from typing import Dict, List
from stochasticx.utils.logging import configure_logger
from stochasticx.utils.spinner_slash import Spinner
import sys
from collections import OrderedDict
import requests

logger = configure_logger(__name__)


def get_docker_client_safely(return_status=False):
    """Get docker client safely"""
    try:
        client = docker.from_env()
        return client
    except:
        if return_status:
            return None
        else:
            logger.error(
                "Cannot connect to the Docker daemon. Make sure Docker is correctly installed and running."
            )
            sys.exit()


def wait_container_is_up(local_url):
    error = True
    while error:
        try:
            response = requests.post(url=local_url, json={})
            if response.status_code != 404:
                error = False
            else:
                time.sleep(10)
                logger.debug(str(response.text))
        except Exception as e:
            logger.debug(str(e))
            time.sleep(10)


def exists_container(container_name: str) -> bool:
    """Check if container already exists

    :param container_name: container name
    :return: if the container exists or not
    """

    client = get_docker_client_safely()
    all_containers = client.containers.list(all=True)

    for container in all_containers:
        if container_name == container.name:
            return True

    return False


def exists_running_container(container_name: str) -> bool:
    client = get_docker_client_safely()

    try:
        container = client.containers.get(container_name)
    except:
        return False

    if container is None:
        return False
    else:
        return container.status == "running"


def start_container(
    docker_image: str,
    ports: Dict[str, str],
    container_name: str,
    detach: bool = True,
    gpu: bool = False,
    volumes: List[str] = [],
    volumes_from: List[str] = [],
    network_links: Dict[str, str] = None,
    shm_size="1G",
):
    """Starts a Docker container locally

    :param docker_image: the Docker image
    :param ports: a dictionary speciying the ports
    :param detach: detach it or not, defaults to True
    :param gpu: use GPUs or not, defaults to False
    :param volumes: list of volumes to use with container
    """

    with Spinner():
        client = get_docker_client_safely()

        if exists_container(container_name):
            # logger.warning("Container {} already running. Stopping and removing it...")
            stop_and_remove_container(container_name)

        device_requests = []
        if gpu:
            device_requests = [
                docker.types.DeviceRequest(device_ids=["all"], capabilities=[["gpu"]])
            ]

        try:
            container = client.containers.run(
                docker_image,
                detach=detach,
                ports=ports,
                device_requests=device_requests,
                name=container_name,
                volumes_from=volumes_from,
                volumes=volumes,
                links=network_links,
                shm_size=shm_size,
            )
        except docker.errors.DockerException as ex:
            logger.error(str(ex))

        return container.id


def stop_and_remove_container(container_name):
    """Stops and removes the container

    :param container_name: container name
    """

    client = get_docker_client_safely()

    try:
        client = get_docker_client_safely()
        container = client.containers.get(container_name)
        container.stop(timeout=50)
        container.remove()
    except docker.errors.DockerException as ex:
        logger.error(str(ex))


def get_logs_container(container_name):
    """Get the logs of a Docker container

    :param container_name: container name
    :return: the logs
    """
    client = get_docker_client_safely()

    if exists_container(container_name):
        return client.containers.get(container_name).logs().decode("utf-8")
    else:
        return "No logs\n"


def get_open_ports_container(container_name):
    """Get open ports of a Docker container

    :param container_name: container name
    :return: container ports
    """
    client = get_docker_client_safely()
    postprocess_ports = {}

    if exists_container(container_name):
        ports = client.containers.get(container_name).ports

        for container_port, host_port in ports.items():
            postprocess_ports[container_port] = host_port[0].get("HostPort")

        return postprocess_ports
    else:
        return {}


def auto_port_selection(port, container_name, selected_ports=[]):
    check = False
    client = get_docker_client_safely()
    port_maps = {}
    for container in client.containers.list(all=True):
        name = container.name
        ports = container.ports
        for host_port in ports.values():
            if host_port is None:
                continue
            host_port = int(host_port[0].get("HostPort"))
            port_maps[host_port] = name

    port_maps = OrderedDict(sorted(port_maps.items()))
    for host_port, name in port_maps.items():
        if port == host_port and name != container_name:
            check = True
            port += 1
    for s_p in selected_ports:
        if s_p == port:
            port += 1

    return port, check



from stochasticx.utils.file_utils import ConversionUtils
from stochasticx.models.models import ModelType
from stochasticx.utils.stat_controller import (
    StatController,
    LocalConversionJobInformation,
)


class ConversionType:
    """Conversion type"""

    ONNX = "onnx"
    TRT = "tensorrt"


class LocalConversion:
    """Model class"""

    def __init__(
        self,
        model_path: str,
        convert_type: str = ConversionType.ONNX,
        model_type: str = ModelType.HUGGINGFACE,
        task_type: str = "",
        convert_params: dict = {},
    ):
        """Initializer

        Args:
            model_path (str): directory path where the model is located
            convert_type (str, optional): conversion type. Defaults to ModelConversion.ONNX.
            model_type (str, optional): model type. Defaults to ModelType.HUGGINGFACE.
            task_type (str, optional): task type.
        """

        self.model_path = model_path
        self.convert_type = convert_type
        self.model_type = model_type
        self.task_type = task_type
        self.convert_params = convert_params
        self.model_id = None
        self.model_info = None
        self.is_uploaded = False

    def convert(self):
        """Convert model to onnx/trt/nvfuser"""

        result = ConversionUtils.convert(
            self.model_path,
            self.convert_type,
            self.model_type,
            self.task_type,
            self.convert_params,
        )
        StatController.add_information(
            LocalConversionJobInformation(
                convert_type=self.convert_type,
                model_type=self.model_type,
                task_type=self.task_type,
                params=self.convert_params,
                job_result=result,
            )
        )
        return result



from dataclasses import dataclass, asdict, field
from re import S
import requests
import click
from typing import Dict, List
from stochasticx.constants.urls import FINETUNING_GET_JOBS_CLOUD_URL
from stochasticx.utils.parse_utils import parse_date
from stochasticx.utils.auth_utils import AuthUtils
from stochasticx.utils.stat_controller import (
    StatController,
    LocalFinetuningJobInformation,
)
from stochasticx.utils.preferences import AppModes, Preferences


@dataclass
class CloudEvaluationResults:
    model_name: str
    evaluation_benchmark: field(default_factory=dict)

    def __str__(self) -> str:
        return """
\t Model name: {}
\t Evaluation benchmarks: {}        
""".format(
            self.model_name, self.evaluation_benchmark
        )


@dataclass
class CloudFinetuningJob:
    id: str
    name: str
    job_status: str = "new"
    created_at: str = ""
    results: List[CloudEvaluationResults] = field(default_factory=list)

    def __str__(self):
        string = """
ID: {}
Name: {}
Status: {}
Created at: {}
Results:
""".format(
            self.id, self.name, self.job_status, self.created_at
        )
        for result in self.results:
            string += result.__str__()

        return string

    def to_table(self):
        columns = ["ID", "Name", "Status", "Created at"]

        values = [self.id, self.name, self.job_status, parse_date(self.created_at)]

        return columns, values


@dataclass
class CloudSequenceClassificationFinetuning(CloudFinetuningJob):
    dataset_id: str = None
    model_id: str = None
    model_source: str = "s3"
    task_type: str = "sequence_classification"
    max_seq_length: int = 128
    sentence1_column: str = "sentence1"
    sentence2_column: str = "sentence2"
    label_column: str = "label"

    def start_job(self):
        data = {
            "dataset": self.dataset_id,
            "jobName": self.name,
            "model": self.model_id,
            "modelSource": self.model_source,
            "selectedColumns": {
                "sentence1_column": self.sentence1_column,
                "sentence2_column": self.sentence2_column,
                "label_column": self.label_column,
            },
            "taskArguments": {"max_seq_length": self.max_seq_length},
            "taskType": self.task_type,
        }

        auth_header = AuthUtils.get_auth_headers()
        response = requests.post(
            FINETUNING_GET_JOBS_CLOUD_URL, json=data, headers=auth_header
        )
        response.raise_for_status()
        data = response.json()

        return data.get("id")


@dataclass
class CloudQuestionAnsweringFinetuning(CloudFinetuningJob):
    dataset_id: str = None
    model_id: str = None
    model_source: str = "s3"
    task_type: str = "question_answering"
    max_seq_length: int = 128
    stride: int = 30
    question_column: str = "question"
    context_column: str = "context"
    answer_column: str = "answers"

    def start_job(self):
        data = {
            "dataset": self.dataset_id,
            "jobName": self.name,
            "model": self.model_id,
            "modelSource": self.model_source,
            "selectedColumns": {
                "question_column": self.question_column,
                "context_column": self.context_column,
                "answer_column": self.answer_column,
            },
            "taskArguments": {"max_seq_length": self.max_seq_length},
            "taskType": self.task_type,
        }

        auth_header = AuthUtils.get_auth_headers()
        response = requests.post(
            FINETUNING_GET_JOBS_CLOUD_URL, json=data, headers=auth_header
        )
        response.raise_for_status()
        data = response.json()

        return data.get("id")


@dataclass
class CloudTokenClassificationFinetuning(CloudFinetuningJob):
    dataset_id: str = None
    model_id: str = None
    model_source: str = "s3"
    task_type: str = "token_classification"
    max_seq_length: int = 128
    label_all_tokens: bool = False
    text_column: str = "tokens"
    label_column: str = "ner_tags"

    def start_job(self):
        data = {
            "dataset": self.dataset_id,
            "jobName": self.name,
            "model": self.model_id,
            "modelSource": self.model_source,
            "selectedColumns": {
                "text_column": self.text_column,
                "label_column": self.label_column,
            },
            "taskArguments": {
                "max_seq_length": self.max_seq_length,
                "label_all_tokens": self.label_all_tokens,
            },
            "taskType": self.task_type,
        }

        auth_header = AuthUtils.get_auth_headers()
        response = requests.post(
            FINETUNING_GET_JOBS_CLOUD_URL, json=data, headers=auth_header
        )
        response.raise_for_status()
        data = response.json()

        return data.get("id")


@dataclass
class CloudSummarizationFinetuning(CloudFinetuningJob):
    dataset_id: str = None
    model_id: str = None
    model_source: str = "s3"
    task_type: str = "summarization"
    max_source_length: int = 256
    max_target_length: int = 32
    lang: str = "en"
    pad_to_max_length: bool = False
    num_beams: int = 4
    ignore_pad_token_for_loss: bool = True
    source_prefix: str = ""
    forced_bos_token: str = None
    text_column: str = "review_body"
    summary_column: str = "review_title"

    def start_job(self):
        data = {
            "dataset": self.dataset_id,
            "jobName": self.name,
            "model": self.model_id,
            "modelSource": self.model_source,
            "selectedColumns": {
                "text_column": self.text_column,
                "summary_column": self.summary_column,
            },
            "taskArguments": {
                "max_source_length": self.max_source_length,
                "max_target_length": self.max_target_length,
                "lang": self.lang,
                "pad_to_max_length": self.pad_to_max_length,
                "num_beams": self.num_beams,
                "ignore_pad_token_for_loss": self.ignore_pad_token_for_loss,
                "source_prefix": self.source_prefix,
                "forced_bos_token": self.forced_bos_token,
            },
            "taskType": self.task_type,
        }

        auth_header = AuthUtils.get_auth_headers()
        response = requests.post(
            FINETUNING_GET_JOBS_CLOUD_URL, json=data, headers=auth_header
        )
        response.raise_for_status()
        data = response.json()

        return data.get("id")


@dataclass
class CloudTranslationFinetuning(CloudFinetuningJob):
    dataset_id: str = None
    model_id: str = None
    model_source: str = "s3"
    task_type: str = "translation"
    max_source_length: int = 128
    max_target_length: int = 128
    src_lang: str = "en"
    tgt_lang: str = "es"
    pad_to_max_length: bool = False
    num_beams: int = 4
    ignore_pad_token_for_loss: bool = True
    source_prefix: str = ""
    forced_bos_token: str = None
    translation_column: str = "translation"

    def start_job(self):
        data = {
            "dataset": self.dataset_id,
            "jobName": self.name,
            "model": self.model_id,
            "modelSource": self.model_source,
            "selectedColumns": {"translation_column": self.translation_column},
            "taskArguments": {
                "max_source_length": self.max_source_length,
                "max_target_length": self.max_target_length,
                "src_lang": self.src_lang,
                "tgt_lang": self.tgt_lang,
                "pad_to_max_length": self.pad_to_max_length,
                "num_beams": self.num_beams,
                "ignore_pad_token_for_loss": self.ignore_pad_token_for_loss,
                "source_prefix": self.source_prefix,
                "forced_bos_token": self.forced_bos_token,
            },
            "taskType": self.task_type,
        }

        auth_header = AuthUtils.get_auth_headers()
        response = requests.post(
            FINETUNING_GET_JOBS_CLOUD_URL, json=data, headers=auth_header
        )
        response.raise_for_status()
        data = response.json()

        return data.get("id")


@dataclass
class CloudTextGenerationFinetuning(CloudFinetuningJob):
    dataset_id: str = None
    model_id: str = None
    model_source: str = "s3"
    task_type: str = "text_generation"
    text_column: str = "review_body"

    def start_job(self):
        data = {
            "dataset": self.dataset_id,
            "jobName": self.name,
            "model": self.model_id,
            "modelSource": self.model_source,
            "selectedColumns": {"text_column": self.text_column},
            "taskArguments": {},
            "taskType": self.task_type,
        }

        auth_header = AuthUtils.get_auth_headers()
        response = requests.post(
            FINETUNING_GET_JOBS_CLOUD_URL, json=data, headers=auth_header
        )
        response.raise_for_status()
        data = response.json()

        return data.get("id")


class CloudFinetuningClient:
    @classmethod
    def get_cloud_finetuning_jobs(cls, fmt="table"):
        auth_header = AuthUtils.get_auth_headers()
        response = requests.get(FINETUNING_GET_JOBS_CLOUD_URL, headers=auth_header)
        response.raise_for_status()

        data = response.json()
        finetuning_jobs = []

        if data.get("data") is not None:
            for job in data.get("data"):
                results = job.get("results")

                evaluation_results = []
                if results is not None and len(results) > 0:
                    for result in results:
                        er = CloudEvaluationResults(
                            model_name=result.get("modelName"),
                            evaluation_benchmark=result.get("evaluationBenchmark"),
                        )

                        evaluation_results.append(er)

                finetuning_job = CloudFinetuningJob(
                    id=job.get("id"),
                    name=job.get("name"),
                    job_status=job.get("jobStatus").get("value"),
                    created_at=job.get("createdAt"),
                    results=evaluation_results,
                )

                finetuning_jobs.append(finetuning_job)

            if fmt == "table":
                columns = []
                values = []

                if len(finetuning_jobs) > 0:
                    columns, _ = finetuning_jobs[0].to_table()

                for job in finetuning_jobs:
                    _, vals = job.to_table()
                    values.append(vals)

                return columns, values

        return finetuning_jobs

    @classmethod
    def get_cloud_finetuning_by_id(cls, id):
        auth_header = AuthUtils.get_auth_headers()
        response = requests.get(
            "{}/{}".format(FINETUNING_GET_JOBS_CLOUD_URL, id), headers=auth_header
        )
        response.raise_for_status()

        data = response.json()

        if data.get("data") is not None:
            job = data.get("data")
            results = job.get("results")

            evaluation_results = []
            if results is not None and len(results) > 0:
                for result in results:
                    er = CloudEvaluationResults(
                        model_name=result.get("modelName"),
                        evaluation_benchmark=result.get("evaluationBenchmark"),
                    )

                    evaluation_results.append(er)

            finetuning_job = CloudFinetuningJob(
                id=job.get("id"),
                name=job.get("name"),
                job_status=job.get("jobStatus").get("value"),
                created_at=job.get("createdAt"),
                results=evaluation_results,
            )

            return finetuning_job

        return None


class LocalFinetuningClient:
    @classmethod
    def get_local_finetuning_jobs(cls, fmt="table"):
        preferences = Preferences.load()
        response = requests.get(f"{preferences.local_finetuning_url}/jobs")
        response.raise_for_status()

        data = response.json()

        if fmt == "table":
            if len(data) > 0:
                columns = ["Name", "status", "Created at"]

                values = []
                for entry in data:
                    values.append(
                        [entry.get("name"), entry.get("status"), entry.get("createdAt")]
                    )

                return columns, values

            else:
                return [], []

    @classmethod
    def get_logs(cls, job_name, fmt="table"):
        preferences = Preferences.load()
        response = requests.get(
            "{}/jobs/{}/logs".format(preferences.local_finetuning_url, job_name)
        )

        try:
            response.raise_for_status()
        except:
            click.secho("\n[+] This job does not exist\n", fg="red", bold=True)

        data = response.json()

        return data.get("logs")


@dataclass
class LocalFinetuning:
    job_name: str
    model_name: str
    dataset_name: str
    num_train_epochs: int = 3
    per_device_train_batch_size: int = 16
    per_device_eval_batch_size: int = 16
    learning_rate: float = 5e-5


@dataclass
class LocalSequenceClassification(LocalFinetuning):
    task_type: str = "sequence_classification"
    max_seq_length: int = 128
    sentence1_column: str = "sentence1"
    sentence2_column: str = "sentence2"
    label_column: str = "label"

    def start_finetuning(self):

        data = asdict(self)
        preferences = Preferences.load()
        response = requests.post(f"{preferences.local_finetuning_url}/jobs", json=data)
        response.raise_for_status()
        StatController.add_information(
            LocalFinetuningJobInformation(
                job_name=self.job_name,
                inference_type=self.task_type,
                model_name=self.model_name,
                dataset_name=self.dataset_name,
                job_result=response.json(),
                params={
                    "num_train_epochs": self.num_train_epochs,
                    "per_device_train_batch_size": self.per_device_train_batch_size,
                    "per_device_eval_batch_size": self.per_device_eval_batch_size,
                    "learning_rate": self.learning_rate,
                    "max_seq_length": self.max_seq_length,
                    "sentence1_column": self.sentence1_column,
                    "sentence2_column": self.sentence2_column,
                    "label_column": self.label_column,
                },
            )
        )


@dataclass
class LocalQuestionAnswering(LocalFinetuning):
    task_type: str = "question_answering"
    max_seq_length: int = 128
    stride: int = 30
    question_column: str = "question"
    context_column: str = "context"
    answer_column: str = "answers"

    def start_finetuning(self):
        data = asdict(self)
        preferences = Preferences.load()
        response = requests.post(f"{preferences.local_finetuning_url}/jobs", json=data)
        response.raise_for_status()

        StatController.add_information(
            LocalFinetuningJobInformation(
                job_name=self.job_name,
                inference_type=self.task_type,
                model_name=self.model_name,
                dataset_name=self.dataset_name,
                job_result=response.json(),
                params={
                    "num_train_epochs": self.num_train_epochs,
                    "per_device_train_batch_size": self.per_device_train_batch_size,
                    "per_device_eval_batch_size": self.per_device_eval_batch_size,
                    "learning_rate": self.learning_rate,
                    "max_seq_length": self.max_seq_length,
                    "stride": self.stride,
                    "question_column": self.question_column,
                    "context_column": self.context_column,
                    "answer_column": self.answer_column,
                },
            )
        )


@dataclass
class LocalTokenClassification(LocalFinetuning):
    task_type: str = "token_classification"
    max_seq_length: int = 128
    label_all_tokens: bool = False
    text_column: str = "tokens"
    label_column: str = "ner_tags"

    def start_finetuning(self):
        data = asdict(self)
        preferences = Preferences.load()
        response = requests.post(f"{preferences.local_finetuning_url}/jobs", json=data)
        response.raise_for_status()

        StatController.add_information(
            LocalFinetuningJobInformation(
                job_name=self.job_name,
                inference_type=self.task_type,
                model_name=self.model_name,
                dataset_name=self.dataset_name,
                job_result=response.json(),
                params={
                    "num_train_epochs": self.num_train_epochs,
                    "per_device_train_batch_size": self.per_device_train_batch_size,
                    "per_device_eval_batch_size": self.per_device_eval_batch_size,
                    "learning_rate": self.learning_rate,
                    "max_seq_length": self.max_seq_length,
                    "label_all_tokens": self.label_all_tokens,
                    "text_column": self.text_column,
                    "label_column": self.label_column,
                },
            )
        )


@dataclass
class LocalSummarization(LocalFinetuning):
    task_type: str = "summarization"
    max_source_length: int = 256
    max_target_length: int = 32
    lang: str = "en"
    pad_to_max_length: bool = False
    num_beams: int = 4
    ignore_pad_token_for_loss: bool = True
    source_prefix: str = ""
    forced_bos_token: str = None
    text_column: str = "review_body"
    summary_column: str = "review_title"

    def start_finetuning(self):
        data = asdict(self)
        preferences = Preferences.load()
        response = requests.post(f"{preferences.local_finetuning_url}/jobs", json=data)
        response.raise_for_status()

        StatController.add_information(
            LocalFinetuningJobInformation(
                job_name=self.job_name,
                inference_type=self.task_type,
                model_name=self.model_name,
                dataset_name=self.dataset_name,
                job_result=response.json(),
                params={
                    "num_train_epochs": self.num_train_epochs,
                    "per_device_train_batch_size": self.per_device_train_batch_size,
                    "per_device_eval_batch_size": self.per_device_eval_batch_size,
                    "learning_rate": self.learning_rate,
                    "max_source_length": self.max_source_length,
                    "max_target_length": self.max_target_length,
                    "lang": self.lang,
                    "pad_to_max_length": self.pad_to_max_length,
                    "num_beams": self.num_beams,
                    "ignore_pad_token_for_loss": self.ignore_pad_token_for_loss,
                    "source_prefix": self.source_prefix,
                    "forced_bos_token": self.forced_bos_token,
                    "text_column": self.text_column,
                    "summary_column": self.summary_column,
                },
            )
        )


@dataclass
class LocalTranslation(LocalFinetuning):
    task_type: str = "translation"
    max_source_length: int = 128
    max_target_length: int = 128
    src_lang: str = "en"
    tgt_lang: str = "es"
    pad_to_max_length: bool = False
    num_beams: int = 4
    ignore_pad_token_for_loss: bool = True
    source_prefix: str = ""
    forced_bos_token: str = None
    translation_column: str = "translation"

    def start_finetuning(self):
        data = asdict(self)
        preferences = Preferences.load()
        response = requests.post(f"{preferences.local_finetuning_url}/jobs", json=data)
        response.raise_for_status()

        StatController.add_information(
            LocalFinetuningJobInformation(
                job_name=self.job_name,
                inference_type=self.task_type,
                model_name=self.model_name,
                dataset_name=self.dataset_name,
                job_result=response.json(),
                params={
                    "num_train_epochs": self.num_train_epochs,
                    "per_device_train_batch_size": self.per_device_train_batch_size,
                    "per_device_eval_batch_size": self.per_device_eval_batch_size,
                    "learning_rate": self.learning_rate,
                    "max_source_length": self.max_source_length,
                    "max_target_length": self.max_target_length,
                    "src_lang": self.src_lang,
                    "tgt_lang": self.tgt_lang,
                    "pad_to_max_length": self.pad_to_max_length,
                    "num_beams": self.num_beams,
                    "ignore_pad_token_for_loss": self.ignore_pad_token_for_loss,
                    "source_prefix": self.source_prefix,
                    "forced_bos_token": self.forced_bos_token,
                    "translation_column": self.translation_column,
                },
            )
        )


@dataclass
class LocalTextGeneration(LocalFinetuning):
    text_column: str = "review_body"

    def start_finetuning(self):
        data = asdict(self)
        preferences = Preferences.load()
        response = requests.post(f"{preferences.local_finetuning_url}/jobs", json=data)
        response.raise_for_status()



import json
import time
from pathlib import Path
import os
import click

from stochasticx.utils.docker import (
    exists_container,
    start_container,
    stop_and_remove_container,
    auto_port_selection,
    wait_container_is_up,
)
from stochasticx.utils.preferences import Preferences, AppModes
from stochasticx.benchmark.local_benchmark import LocalBenchmark
from stochasticx.models.models import Models
from stochasticx.constants.docker_images import DockerImages, ContainerNames
from stochasticx.utils.auth_utils import AuthUtils
from stochasticx.constants.urls import get_local_url, LocalRoutes
import sys

from stochasticx.utils.stat_controller import EventLogger

preferences = Preferences.load()


@click.group(name="benchmarking")
def benchmarking():
    try:
        AuthUtils.get_auth_headers()
    except:
        sys.exit()
    EventLogger.log_event("benchmarking")


@click.command(name="start")
@click.option(
    "--port",
    default=5001,
    help="Port which Stochastic local benchmark container will run on",
)
@click.option(
    "--job_name", required=True, help="Directory where the model to upload is located"
)
@click.option("--model_id", required=True, help="Id of registered model")
@click.option(
    "--task_type",
    required=True,
    help="Task type. It should be sequence_classification, token_classification, question_answering, summarization,translation",
)
@click.option(
    "--task_name",
    required=True,
    help="Name of task: onnx_benchmark, nvfuser_benchmark, tensorrt_benchmark",
)
@click.option(
    "--params_file",
    required=True,
    help="File with params for benchmarking like input information and server config",
)
def benchmarking_start(port, job_name, model_id, task_type, task_name, params_file):
    preferences = Preferences.load()
    if (
        not exists_container(ContainerNames.BENCHMARK)
        and preferences.current_mode == AppModes.LOCAL
    ):
        click.secho(
            "\n[+] We are configuring the environment for model benchmarking. If it is the first time, it will take some minutes\n",
            fg="yellow",
            bold=True,
        )

        original_port = port
        port, is_replace = auto_port_selection(port, ContainerNames.BENCHMARK)
        if is_replace:
            click.secho(
                f"[+] Port {original_port} is using on another service, port {port} will be selected to run stochasticx model benchmark",
                fg="blue",
                bold=True,
            )

        start_container(
            docker_image=DockerImages.BENCHMARK,
            ports={
                "3000": port,
            },
            container_name=ContainerNames.BENCHMARK,
            detach=True,
            gpu=True,
            volumes=["stochasticx:/vol"],
            network_links={"x-local": "x-local"},
        )

        click.echo("[+] Setting up preferences...")
        preferences = Preferences.load()
        preferences.current_mode = AppModes.LOCAL
        preferences.local_benchmark_url = f"http://127.0.0.1:{port}"
        Preferences.save(preferences)

        local_url = get_local_url(LocalRoutes.BENCHMARKING_URL, "local_benchmark_url")

        wait_container_is_up(local_url)

        click.secho(
            f"[+] x-local-benchmarking server running in the port {port}",
            fg="green",
            bold=True,
        )

    local_url = get_local_url(LocalRoutes.BENCHMARKING_URL, "local_benchmark_url")
    model = Models.get_model(model_id, "local")
    with open(params_file) as f:
        params = json.load(f)
    if preferences.current_mode == AppModes.LOCAL:
        benchmark = LocalBenchmark(
            job_name=job_name,
            task_name=task_name,
            model_type=model.model_type,
            model_path=model.directory_path,
            task_type=task_type,
            params=params,
            server_url=local_url,
        )
        click.echo("Benchmarking model...")
        result = benchmark.benchmark().json()
        if benchmark["status"] == "successful":
            click.secho(
                f"\n[+] Benchmark is done, here is the result: \n {result['message']}",
                fg="green",
                bold=True,
            )
        else:
            click.secho(
                f"\n[-] Convertion has failed, the error is:\n {str(result['message'])}",
                fg="red",
                bold=True,
            )

        if exists_container(ContainerNames.BENCHMARK):
            click.secho(
                f"[+] Stopping and removing {ContainerNames.BENCHMARK}",
                fg="blue",
                bold=True,
            )
            stop_and_remove_container(ContainerNames.BENCHMARK)
            click.secho("[+] Removed", fg="green", bold=True)
    EventLogger.log_event("benchmarking_start")


benchmarking.add_command(benchmarking_start)



import time
import click

from stochasticx.finetuning.finetuning import (
    LocalFinetuningClient,
    CloudFinetuningClient,
    LocalQuestionAnswering,
    LocalSequenceClassification,
    CloudSequenceClassificationFinetuning,
    LocalSummarization,
    LocalTextGeneration,
    LocalTokenClassification,
    LocalTranslation,
    LocalQuestionAnswering,
    LocalSummarization,
    LocalTranslation,
    LocalTextGeneration,
)
from stochasticx.utils.parse_utils import print_table
from stochasticx.utils.docker import (
    start_container,
    stop_and_remove_container,
    get_logs_container,
    get_open_ports_container,
    exists_container,
    exists_running_container,
    auto_port_selection,
    wait_container_is_up,
)
from stochasticx.utils.gpu_utils import is_nvidia_gpu_available
from stochasticx.utils.preferences import Preferences, AppModes
from stochasticx.constants.docker_images import DockerImages, ContainerNames
from stochasticx.utils.auth_utils import AuthUtils
import sys
import requests

from stochasticx.utils.stat_controller import EventLogger


@click.group(name="finetuning")
@click.option(
    "--port",
    default=5050,
    help="Port which Stochastic local finetuning container will run on",
)
def finetuning_jobs(port):
    try:
        AuthUtils.get_auth_headers()
    except:
        sys.exit()

    preferences = Preferences.load()

    if (
        not exists_running_container("x-finetuning")
        and preferences.current_mode == AppModes.LOCAL
    ):
        click.secho(
            "\n[+] We are configuring the environment for finetuning. If it is the first time, it will take some minutes\n",
            fg="yellow",
            bold=True,
        )

        original_port = port
        port, is_replace = auto_port_selection(port, ContainerNames.FINETUNE)
        if is_replace:
            click.secho(
                f"[+] Port {original_port} is using on another service, port {port} will be selected to run stochasticx model finetuning",
                fg="blue",
                bold=True,
            )

        start_container(
            docker_image=DockerImages.FINETUNE,
            ports={"5000": str(port)},
            container_name=ContainerNames.FINETUNE,
            detach=True,
            gpu=is_nvidia_gpu_available(),
            volumes=["stochasticx:/vol"],
            network_links={"x-local": "x-local"},
        )

        click.secho("[+] Environment already configured\n", fg="green", bold=True)
        click.echo("[+] Setting up preferences...")
        preferences = Preferences.load()
        preferences.current_mode = AppModes.LOCAL
        preferences.local_finetuning_url = f"http://127.0.0.1:{port}"
        Preferences.save(preferences)

        wait_container_is_up(f"{preferences.local_finetuning_url}/jobs")

        click.secho(
            f"[+] x-local-finetuning server running in the port {port}",
            fg="green",
            bold=True,
        )
        EventLogger.log_event("finetuning_local")


@click.command(name="ls")
def ls_finetuning_jobs():
    preferences = Preferences.load()

    click.secho("\n[+] Collecting all jobs\n", fg="blue", bold=True)

    if preferences.current_mode == AppModes.LOCAL:
        columns, values = LocalFinetuningClient.get_local_finetuning_jobs(fmt="table")
        EventLogger.log_event("finetuning_local_ls")
    else:
        columns, values = CloudFinetuningClient.get_cloud_finetuning_jobs(fmt="table")
        EventLogger.log_event("finetuning_cloud_ls")

    print_table(columns, values)


@click.command(name="inspect")
@click.option("--id", required=True, help="Finetuning job ID or job name")
def inspect_finetuning_jobs(id):
    preferences = Preferences.load()

    click.secho("\n[+] Inspecting the specified job\n", fg="blue", bold=True)

    if preferences.current_mode == AppModes.LOCAL:
        # click.secho("\n[+] This command is not support for local mode\n", fg='red', bold=True)
        preferences = Preferences.load()
        response = requests.get(f"{preferences.local_finetuning_url}/jobs/{id}").json()
        if response["status"] == "running":
            click.secho("\n[+] Job is still running\n", fg="green", bold=True)
        elif response["status"] == "successful":
            logs = LocalFinetuningClient.get_logs(id)
            click.secho(
                f"\n[+] Job is successful, here is the logs:\n{logs}",
                fg="green",
                bold=True,
            )
        elif response["status"] == "failed":
            logs = LocalFinetuningClient.get_logs(id)
            click.secho(f"\n[-] Job is failed\n", fg="red", bold=True)
        EventLogger.log_event("finetuning_local_inspect")
    else:
        job = CloudFinetuningClient.get_cloud_finetuning_by_id(id)
        click.secho(str(job), fg="white", bold=True)
        EventLogger.log_event("finetuning_cloud_inspect")


@click.command(name="logs")
@click.option("--name", required=True, help="Job name")
def logs_finetuning_job(name):
    click.secho("\n[+] Collecting logs\n", fg="blue", bold=True)
    logs = LocalFinetuningClient.get_logs(name)
    click.secho(logs, fg="white", bold=True)
    EventLogger.log_event("finetuning_logs")


@click.group(name="launch")
def job_launch():
    pass


@click.command(name="sequence_classification")
@click.option("--name", required=True, help="Job name")
@click.option("--model_name_or_id", required=True, help="Model ID")
@click.option("--dataset_name_or_id", required=True, help="Dataset ID")
@click.option("--sentence1_column", required=True, help="Sentence 1")
@click.option("--sentence2_column", required=True, help="Sentence 2")
@click.option("--label_column", required=True, help="Label")
@click.option(
    "--max_seq_length", required=False, default=128, help="Maximum sequence length"
)
@click.option(
    "--num_train_epochs",
    required=False,
    default=3,
    show_default=True,
    help="Number of training epochs",
)
@click.option(
    "--per_device_train_batch_size",
    required=False,
    default=16,
    show_default=True,
    help="Train batch size",
)
@click.option(
    "--learning_rate",
    required=False,
    default=5e-5,
    show_default=True,
    help="Learning rate",
)
def job_launch_sequence_classification(
    name,
    model_name_or_id,
    dataset_name_or_id,
    sentence1_column,
    sentence2_column,
    label_column,
    max_seq_length,
    num_train_epochs,
    per_device_train_batch_size,
    learning_rate,
):
    preferences = Preferences.load()

    if preferences.current_mode == AppModes.LOCAL:
        local_finetuning = LocalSequenceClassification(
            job_name=name,
            model_name=model_name_or_id,
            dataset_name=dataset_name_or_id,
            sentence1_column=sentence1_column,
            sentence2_column=sentence2_column,
            label_column=label_column,
            max_seq_length=max_seq_length,
            num_train_epochs=num_train_epochs,
            per_device_eval_batch_size=per_device_train_batch_size,
            learning_rate=learning_rate,
        )

        click.secho("\n[+] Launching finetuning job\n", fg="blue", bold=True)
        local_finetuning.start_finetuning()
        EventLogger.log_event("finetuning_local_sequence_classification")
    else:
        cloud_finetuning = CloudSequenceClassificationFinetuning(
            id=None,
            name=name,
            dataset_id=dataset_name_or_id,
            model_id=model_name_or_id,
            sentence1_column=sentence1_column,
            sentence2_column=sentence2_column,
            label_column=label_column,
            max_seq_length=max_seq_length,
        )

        click.secho("\n[+] Launching finetuning job\n", fg="blue", bold=True)
        cloud_finetuning.start_job()
        EventLogger.log_event("finetuning_cloud_sequence_classification")


@click.command(name="question_answering")
@click.option("--name", required=True, help="Job name")
@click.option("--model_name_or_id", required=True, help="Model ID")
@click.option("--dataset_name_or_id", required=True, help="Dataset ID")
@click.option("--question_column", required=True, help="Question column")
@click.option("--context_column", required=True, help="Context column")
@click.option("--answer_column", required=True, help="Answer column")
@click.option(
    "--max_seq_length", required=False, default=128, help="Maximum sequence length"
)
@click.option("--stride", required=False, default=30, help="Stride")
@click.option(
    "--num_train_epochs",
    required=False,
    default=3,
    show_default=True,
    help="Number of training epochs",
)
@click.option(
    "--per_device_train_batch_size",
    required=False,
    default=16,
    show_default=True,
    help="Train batch size",
)
@click.option(
    "--learning_rate",
    required=False,
    default=5e-5,
    show_default=True,
    help="Learning rate",
)
def job_launch_question_answering(
    name,
    model_name_or_id,
    dataset_name_or_id,
    question_column,
    context_column,
    answer_column,
    max_seq_length,
    stride,
    num_train_epochs,
    per_device_train_batch_size,
    learning_rate,
):
    preferences = Preferences.load()

    if preferences.current_mode == AppModes.LOCAL:
        local_finetuning = LocalQuestionAnswering(
            job_name=name,
            model_name=model_name_or_id,
            dataset_name=dataset_name_or_id,
            question_column=question_column,
            context_column=context_column,
            answer_column=answer_column,
            max_seq_length=max_seq_length,
            stride=stride,
            num_train_epochs=num_train_epochs,
            per_device_eval_batch_size=per_device_train_batch_size,
            learning_rate=learning_rate,
        )

        click.secho("\n[+] Launching finetuning job\n", fg="blue", bold=True)
        local_finetuning.start_finetuning()
        EventLogger.log_event("finetuning_local_question_answering")


@click.command(name="token_classification")
@click.option("--name", required=True, help="Job name")
@click.option("--model_name_or_id", required=True, help="Model ID")
@click.option("--dataset_name_or_id", required=True, help="Dataset ID")
@click.option("--text_column", required=True, help="Text column")
@click.option("--label_column", required=True, help="Label column")
@click.option(
    "--max_seq_length", required=False, default=128, help="Maximum sequence length"
)
@click.option(
    "--label_all_tokens", required=False, default=False, help="Label all tokens"
)
@click.option(
    "--num_train_epochs",
    required=False,
    default=3,
    show_default=True,
    help="Number of training epochs",
)
@click.option(
    "--per_device_train_batch_size",
    required=False,
    default=16,
    show_default=True,
    help="Train batch size",
)
@click.option(
    "--learning_rate",
    required=False,
    default=5e-5,
    show_default=True,
    help="Learning rate",
)
def job_launch_token_classification(
    name,
    model_name_or_id,
    dataset_name_or_id,
    text_column,
    label_column,
    max_seq_length,
    label_all_tokens,
    num_train_epochs,
    per_device_train_batch_size,
    learning_rate,
):
    preferences = Preferences.load()

    if preferences.current_mode == AppModes.LOCAL:
        local_finetuning = LocalTokenClassification(
            job_name=name,
            model_name=model_name_or_id,
            dataset_name=dataset_name_or_id,
            text_column=text_column,
            label_column=label_column,
            max_seq_length=max_seq_length,
            label_all_tokens=label_all_tokens,
            num_train_epochs=num_train_epochs,
            per_device_eval_batch_size=per_device_train_batch_size,
            learning_rate=learning_rate,
        )

        click.secho("\n[+] Launching finetuning job\n", fg="blue", bold=True)
        local_finetuning.start_finetuning()
        EventLogger.log_event("finetuning_local_token_classification")


@click.command(name="summarization")
@click.option("--name", required=True, help="Job name")
@click.option("--model_name_or_id", required=True, help="Model ID")
@click.option("--dataset_name_or_id", required=True, help="Dataset ID")
@click.option("--text_column", required=True, help="Text column")
@click.option("--summary_column", required=True, help="Summary column")
@click.option(
    "--max_source_length",
    required=False,
    default=256,
    help="Maximum source sequence length",
)
@click.option(
    "--max_target_length",
    required=False,
    default=32,
    help="Maximum target sequence length",
)
@click.option("--lang", required=False, default="en", help="Language")
@click.option(
    "--pad_to_max_length", required=False, default=False, help="Pad to max length"
)
@click.option("--num_beams", required=False, default=4, help="Number of beams")
@click.option(
    "--ignore_pad_token_for_loss",
    required=False,
    default=True,
    help="Ignore pad token for loss",
)
@click.option("--source_prefix", required=False, default="", help="Source prefix")
@click.option(
    "--forced_bos_token", required=False, default=None, help="Forced BOS token"
)
@click.option(
    "--num_train_epochs",
    required=False,
    default=3,
    show_default=True,
    help="Number of training epochs",
)
@click.option(
    "--per_device_train_batch_size",
    required=False,
    default=16,
    show_default=True,
    help="Train batch size",
)
@click.option(
    "--learning_rate",
    required=False,
    default=5e-5,
    show_default=True,
    help="Learning rate",
)
def job_launch_summarization(
    name,
    model_name_or_id,
    dataset_name_or_id,
    text_column,
    summary_column,
    max_source_length,
    max_target_length,
    lang,
    pad_to_max_length,
    num_beams,
    ignore_pad_token_for_loss,
    source_prefix,
    forced_bos_token,
    num_train_epochs,
    per_device_train_batch_size,
    learning_rate,
):
    preferences = Preferences.load()

    if preferences.current_mode == AppModes.LOCAL:
        local_finetuning = LocalSummarization(
            job_name=name,
            model_name=model_name_or_id,
            dataset_name=dataset_name_or_id,
            text_column=text_column,
            max_source_length=max_source_length,
            max_target_length=max_target_length,
            lang=lang,
            pad_to_max_length=pad_to_max_length,
            num_beams=num_beams,
            ignore_pad_token_for_loss=ignore_pad_token_for_loss,
            source_prefix=source_prefix,
            forced_bos_token=forced_bos_token,
            num_train_epochs=num_train_epochs,
            per_device_eval_batch_size=per_device_train_batch_size,
            learning_rate=learning_rate,
        )

        click.secho("\n[+] Launching finetuning job\n", fg="blue", bold=True)
        local_finetuning.start_finetuning()
        EventLogger.log_event("finetuning_local_token_summarization")


@click.command(name="translation")
@click.option("--name", required=True, help="Job name")
@click.option("--model_name_or_id", required=True, help="Model ID")
@click.option("--dataset_name_or_id", required=True, help="Dataset ID")
@click.option("--translation_column", required=True, help="Translation column")
@click.option(
    "--max_source_length",
    required=False,
    default=128,
    help="Maximum source sequence length",
)
@click.option(
    "--max_target_length",
    required=False,
    default=128,
    help="Maximum target sequence length",
)
@click.option("--src_lang", required=False, default="en", help="Source language")
@click.option("--tgt_lang", required=False, default="es", help="Target language")
@click.option(
    "--pad_to_max_length", required=False, default=False, help="Pad to max length"
)
@click.option("--num_beams", required=False, default=4, help="Number of beams")
@click.option(
    "--ignore_pad_token_for_loss",
    required=False,
    default=True,
    help="Ignore pad token for loss",
)
@click.option("--source_prefix", required=False, default="", help="Source prefix")
@click.option(
    "--forced_bos_token", required=False, default=None, help="Forced BOS token"
)
@click.option(
    "--num_train_epochs",
    required=False,
    default=3,
    show_default=True,
    help="Number of training epochs",
)
@click.option(
    "--per_device_train_batch_size",
    required=False,
    default=16,
    show_default=True,
    help="Train batch size",
)
@click.option(
    "--learning_rate",
    required=False,
    default=5e-5,
    show_default=True,
    help="Learning rate",
)
def job_launch_translation(
    name,
    model_name_or_id,
    dataset_name_or_id,
    translation_column,
    max_source_length,
    max_target_length,
    src_lang,
    tgt_lang,
    pad_to_max_length,
    num_beams,
    ignore_pad_token_for_loss,
    source_prefix,
    forced_bos_token,
    num_train_epochs,
    per_device_train_batch_size,
    learning_rate,
):
    preferences = Preferences.load()

    if preferences.current_mode == AppModes.LOCAL:
        local_finetuning = LocalTranslation(
            job_name=name,
            model_name=model_name_or_id,
            dataset_name=dataset_name_or_id,
            text_column=translation_column,
            max_source_length=max_source_length,
            max_target_length=max_target_length,
            src_lang=src_lang,
            tgt_lang=tgt_lang,
            pad_to_max_length=pad_to_max_length,
            num_beams=num_beams,
            ignore_pad_token_for_loss=ignore_pad_token_for_loss,
            source_prefix=source_prefix,
            forced_bos_token=forced_bos_token,
            num_train_epochs=num_train_epochs,
            per_device_eval_batch_size=per_device_train_batch_size,
            learning_rate=learning_rate,
        )

        click.secho("\n[+] Launching finetuning job\n", fg="blue", bold=True)
        local_finetuning.start_finetuning()
        EventLogger.log_event("finetuning_local_translation")


@click.command(name="text_generation")
@click.option("--name", required=True, help="Job name")
@click.option("--model_name_or_id", required=True, help="Model ID")
@click.option("--dataset_name_or_id", required=True, help="Dataset ID")
@click.option("--text_column", required=True, help="Text column")
@click.option(
    "--num_train_epochs",
    required=False,
    default=3,
    show_default=True,
    help="Number of training epochs",
)
@click.option(
    "--per_device_train_batch_size",
    required=False,
    default=16,
    show_default=True,
    help="Train batch size",
)
@click.option(
    "--learning_rate",
    required=False,
    default=5e-5,
    show_default=True,
    help="Learning rate",
)
def job_launch_text_generation(
    name,
    model_name_or_id,
    dataset_name_or_id,
    text_column,
    num_train_epochs,
    per_device_train_batch_size,
    learning_rate,
):
    preferences = Preferences.load()

    if preferences.current_mode == AppModes.LOCAL:
        local_finetuning = LocalTextGeneration(
            job_name=name,
            model_name=model_name_or_id,
            dataset_name=dataset_name_or_id,
            text_column=text_column,
            num_train_epochs=num_train_epochs,
            per_device_eval_batch_size=per_device_train_batch_size,
            learning_rate=learning_rate,
        )

        click.secho("\n[+] Launching finetuning job\n", fg="blue", bold=True)
        local_finetuning.start_finetuning()
        EventLogger.log_event("finetuning_local_text_generation")


job_launch.add_command(job_launch_sequence_classification)
job_launch.add_command(job_launch_question_answering)
job_launch.add_command(job_launch_summarization)
job_launch.add_command(job_launch_translation)
job_launch.add_command(job_launch_token_classification)
job_launch.add_command(job_launch_text_generation)
finetuning_jobs.add_command(ls_finetuning_jobs)
finetuning_jobs.add_command(logs_finetuning_job)
finetuning_jobs.add_command(job_launch)
finetuning_jobs.add_command(inspect_finetuning_jobs)


"""

stochasticx finetuning launch sequence_classification \
    --name test2 \
    --model_id bert-base-uncased \
    --dataset_id "/home/azureuser/datasets/mrpc" \
    --sentence1_column "sentence1" \
    --sentence2_column "sentence2" \
    --label_column "label"

"""



import os
import json
from pathlib import Path
from enum import Enum
from pydantic import BaseModel

if os.name == 'nt':
  HOME = "%userprofile%"
else:
  HOME = "$HOME"

PREFERENCES_DIR =  os.path.expandvars(f"{HOME}/.stochastic")
PREFERENCES_PATH = os.path.expandvars(f"{HOME}/.stochastic/preferences.json")


class AppModes:
    LOCAL = "local"
    CLOUD = "cloud"


class Defaults:
    CLOUD_URL_PREFIX = "https://api.stochastic.ai"
    LOCAL_URL_PREFIX = "http://127.0.0.1:3000"
    LOCAL_CONVERSION_URL = "http://localhost:5000"
    LOCAL_FINETUNING_URL = "http://localhost:5050"
    LOCAL_INFERENCE_URL = "http://localhost:9000"
    LOCAL_DEPLOYMENT_URL = "http://localhost:5051"
    LOCAL_BENCHMARK_URL = "http://localhost:5001"


class PreferencesItem(BaseModel):
    cloud_url: str = Defaults.CLOUD_URL_PREFIX
    local_url: str = Defaults.LOCAL_URL_PREFIX
    local_conversion_url: str = Defaults.LOCAL_CONVERSION_URL
    local_finetuning_url: str = Defaults.LOCAL_FINETUNING_URL
    local_inference_url: str = Defaults.LOCAL_INFERENCE_URL
    local_benchmark_url: str = Defaults.LOCAL_BENCHMARK_URL
    local_deployment_url: str = Defaults.LOCAL_DEPLOYMENT_URL
    current_mode: str = AppModes.CLOUD


class Preferences:
    """JSON config file representation in preferences"""

    @staticmethod
    def exists() -> bool:
        if Path(PREFERENCES_PATH).exists():
            return True
        return False

    @staticmethod
    def load() -> PreferencesItem:
        try:
            with open(PREFERENCES_PATH) as f:
                data = json.load(f)
            return PreferencesItem(**data)
        except Exception as e:
            # print("ERROR loading preferences", e)
            return PreferencesItem()

    @staticmethod
    def save(updated_preferences: PreferencesItem) -> None:
        if not Path(PREFERENCES_DIR).exists():
            os.makedirs(PREFERENCES_DIR)

        with open(PREFERENCES_PATH, "w") as f:
            f.write(updated_preferences.json())



from rich.console import Console
from rich.table import Table

import datetime


def parse_date(date_string):
    source_fmt = "%Y-%m-%dT%H:%M:%S.%fZ"
    return str(
        datetime.datetime.strptime(date_string, source_fmt).replace(microsecond=0)
    )


def print_table(columns, list_of_list_values):
    table = Table(title=None)

    for column in columns:
        table.add_column(column)

    for values in list_of_list_values:
        table.add_row(*values)

    console = Console()
    console.print(table)



import sys
import requests

from stochasticx.constants.urls import LocalRoutes
from stochasticx.utils.stat_controller import (
    StatController,
    LocalBenchmarkJobInformation,
)


class LocalBenchmark:
    def __init__(
        self,
        job_name: str,
        task_name: str,
        model_type: str,
        model_path: str,
        task_type: str,
        params: dict,
        server_url: str = LocalRoutes.BENCHMARKING_URL,
    ):
        self.job_name = job_name
        self.task_name = task_name
        self.model_type = model_type
        self.model_path = model_path
        self.task_type = task_type
        self.params = params
        self.server_url = server_url

    def benchmark(self, **kwargs):
        """Benchmark the model"""
        result = requests.post(
            url=self.server_url,
            json={
                "job_name": self.job_name,
                "task_type": self.task_type,
                "task_name": self.task_name,
                "model_type": self.model_type,
                "model_path": self.model_path,
                "params": self.params,
            },
        )
        print(result.text)
        StatController.add_information(
            LocalBenchmarkJobInformation(
                job_name=self.job_name,
                task_type=self.task_type,
                task_name=self.task_name,
                model_type=self.model_type,
                params=self.params,
                job_result=result.json(),
            )
        )

        return result

    def __call__(self, **kwargs):
        """It is equivalent to the benchmark method.

        :return: the outputs of the model
        """
        return self.benchmark(**kwargs)



from stochasticx.deployment.deployments import LocalDeploymentsClient, DeploymentsClient
from stochasticx.utils.logging import configure_logger
from stochasticx.utils.preferences import Preferences, AppModes
from stochasticx.utils.stat_controller import (
    StatController,
    LocalInferenceJobInformation,
)

import requests

logger = configure_logger(__name__)


class InferenceStatus:
    ALIVE = "alive"
    READY = "ready"
    DOWN = "down"


class InferenceModel:
    def __init__(self, deployment_id):
        self.deployment_id = deployment_id

        preferences = Preferences.load()

        if preferences.current_mode == AppModes.LOCAL:
            self.deployment = LocalDeploymentsClient.get_deployment_by_id(
                self.deployment_id
            )
        else:
            self.deployment = DeploymentsClient.get_deployment_by_id(
                self.deployment_id
            )

    def inference(self, *args, **kwargs):
        return """
    URL: {}
    API Key: {}
""".format(self.deployment.client_url, self.deployment.api_key)


class SequenceClassificationModel(InferenceModel):
    def __init__(self, deployment_id):
        super().__init__(deployment_id)

    def format_inputs_local(self, texts):
        return {
            "inputs": [
                {"name": "text", "datatype": "BYTES", "shape": [1], "data": texts}
            ]
        }

    def format_inputs(self, texts):
        return {
            "inputs": [
                {
                    "name": "text",
                    "datatype": "BYTES",
                    "shape": [1, len(texts)],
                    "data": texts,
                }
            ]
        }

    def format_output(self, outputs):
        outputs = outputs.get("outputs")
        if outputs is not None and isinstance(outputs, list):
            labels, scores = outputs[0], outputs[1]
            return labels.get("data"), scores.get("data")

    def local_inference(self, texts):
        model_inputs = self.format_inputs_local(texts)

        preferences = Preferences.load()
        endpoint_url = "{}/v2/models/{}_{}/versions/1/infer".format(
            preferences.local_inference_url,
            self.deployment.model_name,
            self.deployment.type,
        )

        r = requests.post(endpoint_url, json=model_inputs)
        r.raise_for_status()

        StatController.add_local_inference_job(
            LocalInferenceJobInformation(
                inference_type="sequence_classification",
                model_name=self.deployment.model_name,
                model_type=self.deployment.type,
                job_result=r.json(),
            )
        )

        outputs = r.json()
        labels, scores = self.format_output(outputs)

        return labels, scores


class QuestionAnsweringModel(InferenceModel):
    def __init__(self, deployment_id):
        super().__init__(deployment_id)

    def format_inputs_local(self, questions, contexts):
        return {
            "inputs": [
                {
                    "name": "question",
                    "datatype": "BYTES",
                    "shape": [1],
                    "data": questions,
                },
                {"name": "text", "datatype": "BYTES", "shape": [1], "data": contexts},
            ]
        }

    def format_inputs(self, questions, contexts):
        return {
            "inputs": [
                {
                    "name": "question",
                    "datatype": "BYTES",
                    "shape": [1, len(questions)],
                    "data": questions,
                },
                {
                    "name": "text",
                    "datatype": "BYTES",
                    "shape": [1, len(contexts)],
                    "data": contexts,
                },
            ]
        }

    def format_output(self, outputs):
        outputs = outputs.get("outputs")
        if outputs is not None and isinstance(outputs, list):
            answers = outputs[0]
            return answers.get("data")

    def local_inference(self, questions, contexts):
        model_inputs = self.format_inputs_local(questions, contexts)

        preferences = Preferences.load()
        endpoint_url = "{}/v2/models/{}_{}/versions/1/infer".format(
            preferences.local_inference_url,
            self.deployment.model_name,
            self.deployment.type,
        )

        r = requests.post(endpoint_url, json=model_inputs)
        r.raise_for_status()

        StatController.add_information(
            LocalInferenceJobInformation(
                inference_type="question_answering",
                model_name=self.deployment.model_name,
                model_type=self.deployment.type,
                job_result=r.json(),
            )
        )

        outputs = r.json()
        answers = self.format_output(outputs)

        return answers


class SummarizationModel(InferenceModel):
    def __init__(self, deployment_id):
        super().__init__(deployment_id)

    def format_inputs(self, texts, min_lengths, max_lengths):
        return {
            "inputs": [
                {
                    "name": "text",
                    "datatype": "BYTES",
                    "shape": [1, len(texts)],
                    "data": texts,
                },
                {
                    "name": "min_length",
                    "datatype": "INT32",
                    "shape": [1, len(min_lengths)],
                    "data": min_lengths,
                },
                {
                    "name": "max_length",
                    "datatype": "INT32",
                    "shape": [1, len(max_lengths)],
                    "data": max_lengths,
                },
            ]
        }

    def format_inputs_local(self, texts, min_lengths, max_lengths):
        return {
            "inputs": [
                {"name": "text", "datatype": "BYTES", "shape": [1], "data": texts},
                {
                    "name": "min_length",
                    "datatype": "INT32",
                    "shape": [1],
                    "data": min_lengths,
                },
                {
                    "name": "max_length",
                    "datatype": "INT32",
                    "shape": [1],
                    "data": max_lengths,
                },
            ]
        }

    def format_output(self, outputs):
        outputs = outputs.get("outputs")
        if outputs is not None and isinstance(outputs, list):
            summaries = outputs[0]
            return summaries.get("data")

    def local_inference(self, texts, min_lengths, max_lengths):
        model_inputs = self.format_inputs_local(texts, min_lengths, max_lengths)

        preferences = Preferences.load()
        endpoint_url = "{}/v2/models/{}_{}/versions/1/infer".format(
            preferences.local_inference_url,
            self.deployment.model_name,
            self.deployment.type,
        )

        r = requests.post(endpoint_url, json=model_inputs)
        r.raise_for_status()

        StatController.add_information(
            LocalInferenceJobInformation(
                inference_type="summarization",
                model_name=self.deployment.model_name,
                model_type=self.deployment.type,
                job_result=r.json(),
            )
        )

        outputs = r.json()
        summaries = self.format_output(outputs)

        return summaries


class TranslationModel(InferenceModel):
    def __init__(self, deployment_id):
        super().__init__(deployment_id)

    def format_inputs(self, texts, max_lengths):
        return {
            "inputs": [
                {
                    "name": "text",
                    "datatype": "BYTES",
                    "shape": [1, len(texts)],
                    "data": texts,
                },
                {
                    "name": "max_length",
                    "datatype": "INT32",
                    "shape": [1, len(max_lengths)],
                    "data": max_lengths,
                },
            ]
        }

    def format_inputs_local(self, texts, max_lengths):
        return {
            "inputs": [
                {"name": "text", "datatype": "BYTES", "shape": [1], "data": texts},
                {
                    "name": "max_length",
                    "datatype": "INT32",
                    "shape": [1],
                    "data": max_lengths,
                },
            ]
        }

    def format_output(self, outputs):
        outputs = outputs.get("outputs")
        if outputs is not None and isinstance(outputs, list):
            translations = outputs[0]
            return translations.get("data")

    def local_inference(self, texts, max_lengths):
        model_inputs = self.format_inputs_local(texts, max_lengths)

        preferences = Preferences.load()
        endpoint_url = "{}/v2/models/{}_{}/versions/1/infer".format(
            preferences.local_inference_url,
            self.deployment.model_name,
            self.deployment.type,
        )

        r = requests.post(endpoint_url, json=model_inputs)
        r.raise_for_status()

        StatController.add_information(
            LocalInferenceJobInformation(
                inference_type="translation",
                model_name=self.deployment.model_name,
                model_type=self.deployment.type,
                job_result=r.json(),
            )
        )

        outputs = r.json()
        translations = self.format_output(outputs)

        return translations


class TokenClassificationModel(InferenceModel):
    def __init__(self, deployment_id):
        super().__init__(deployment_id)

    def format_inputs(self, texts):
        return {
            "inputs": [
                {
                    "name": "text",
                    "datatype": "BYTES",
                    "shape": [1, len(texts)],
                    "data": texts,
                }
            ]
        }

    def format_inputs_local(self, texts):
        return {
            "inputs": [
                {"name": "text", "datatype": "BYTES", "shape": [1], "data": texts}
            ]
        }

    def format_output(self, outputs):
        outputs = outputs.get("outputs")
        if outputs is not None and isinstance(outputs, list):
            tokens, tags, scores = outputs[0], outputs[1], outputs[2]
            return tokens.get("data"), tags.get("data"), scores.get("data")

    def local_inference(self, texts):
        model_inputs = self.format_inputs_local(texts)

        preferences = Preferences.load()
        endpoint_url = "{}/v2/models/{}_{}/versions/1/infer".format(
            preferences.local_inference_url,
            self.deployment.model_name,
            self.deployment.type,
        )

        r = requests.post(endpoint_url, json=model_inputs)
        r.raise_for_status()

        StatController.add_information(
            LocalInferenceJobInformation(
                inference_type="token_classification",
                model_name=self.deployment.model_name,
                model_type=self.deployment.type,
                job_result=r.json(),
            )
        )

        outputs = r.json()

        return self.format_output(outputs)



import click
from stochasticx.utils.preferences import Preferences, AppModes
from stochasticx.utils.stat_controller import EventLogger


@click.group(name="config")
def config_command():
    pass


@click.command(name="inspect")
def inspect_command():
    preferences = Preferences.load()
    click.secho(
        "[+] You are in the **{}** mode...".format(preferences.current_mode), bold=True
    )
    EventLogger.log_event("config_inspect")


@click.command(name="local")
def local_command():
    click.secho("[+] Setting up **local** mode...", bold=True)
    preferences = Preferences.load()
    preferences.current_mode = AppModes.LOCAL
    Preferences.save(preferences)
    EventLogger.log_event("config_local")


@click.command(name="cloud")
def remote_command():
    click.secho("[+] Setting up **cloud** mode...", bold=True)
    preferences = Preferences.load()
    preferences.current_mode = AppModes.CLOUD
    Preferences.save(preferences)
    EventLogger.log_event("config_cloud")


config_command.add_command(inspect_command)
config_command.add_command(local_command)
config_command.add_command(remote_command)



import setuptools
from pathlib import Path


def add_alias_command(source_command, alias):
    bashrc_path = Path("~/.bashrc").resolve()

    if bashrc_path.exists():
        with open(str(bashrc_path), "a") as bashrc_file:
            bashrc_file.write("\n\n{}={}\n\n".format(source_command, alias))


# add_alias_command("stochasticx", "x")

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

install_requires = [
   "requests==2.28.1",
   "jwt==1.3.1",
   "Click==8.1.3",
   "tqdm==4.64.1",
   "ipywidgets==8.0.2",
   "tritonclient[all]==2.27.0",
   "docker==6.0.1",
   "numpy==1.23.4",
   "Pillow==9.3.0",
   "diffusers==0.11.1",
   "transformers==4.25.1",
   "rich==12.6.0",
   #"psutil==5.9.4",
   "accelerate==0.14.0",
   "pydantic==1.10.2",
   "pandas==1.5.1"
]

extras_require = {"dev": ["pre-commit==2.20.0"]}

"""scripts=[
   "bin/stochasticx"
]"""

setuptools.setup(
   name='stochasticx',
   version='0.0.19',
   author='Marcos Rivera Martínez, Sarthak Langde, Glenn Ko, Subhash G N, Jolina Li',
   author_email='marcos.rm@stochastic.ai, sarthak.langde@stochastic.ai, glenn@stochastic.ai, subhash.gn@stochastic.ai, jolina.li@mail.utoronto.ca',
   description='Stochastic client library',
   long_description=long_description,
   long_description_content_type="text/markdown",
   url="https://github.com/stochasticai/stochasticx",
   classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
    install_requires=install_requires,
    extras_require=extras_require,
    entry_points="""
      [console_scripts]
      stochasticx=stochasticx.scripts.stochasticx:cli
   """,
)
